// Game of Life.cpp : Defines the entry point for the console application.
//
#include "common.h"
#include "life.h"

Grid map;                // square array holding cells
Gridcount numNeighbors;  // square array holding neighbor counts
List newlive,            // cells that have just been vivified
     newdie,             // cells that have just died
     maylive,            // candidates to vivify in next generation
     maydie;             // candidates to kill in next generation
char *fName = NULL;      // pointer to file name string from argv
int gens;                // number of generations to be simulated
Cell c1, c2;             // boundary points for final print


int main(int argc, char* argv[])
{
   int gencount=0;
   if (argc == 2) 
	   fName = argv[1];
   else {
	   usage();
	   exit(1);
   }

   printf("FILE NAME: %s\n\n",fName);

   Initialize(map, numNeighbors, &newlive, &newdie, &maylive, &maydie, fName, &gens, &c1, &c2);
   while (gencount < gens) {
      gencount++;
      TraverseList(&maylive, Vivify);
      TraverseList(&maydie, Kill);
      ClearList(&maylive);
      ClearList(&maydie);
      TraverseList(&newlive, AddNeighbors);
      TraverseList(&newdie, SubtractNeighbors);
      ClearList(&newlive);
      ClearList(&newdie);
   }
   WriteMap(map, gencount, c1, c2);
   return 0;
}

void AddNeighbors(Cell cell)
{// Check neighbors around cell that has recently come to life  
 // Update neighbor counts of cells around cell of interest; add to appropriate list
   int nbrrow,              // loop index for row of neighbor loops
       nbrcol;              // column loop index
   Cell neighbor;           // structure form of a neighbor

   for (nbrrow = cell.row-1; nbrrow <=cell.row+1; nbrrow++)
      for (nbrcol = cell.col-1; nbrcol <=cell.col+1; nbrcol++)
         if (nbrrow != cell.row || nbrcol != cell.col) {  //skip cell itself
            numNeighbors[nbrrow][nbrcol]++;
            switch(numNeighbors[nbrrow][nbrcol]) {
               case 0:
                  Error("Impossible case in AddNeighbors.");
                  break;
               case 3: 
                  if (map[nbrrow][nbrcol] == DEAD) {
                     neighbor.row = nbrrow;
                     neighbor.col = nbrcol;
                     AddList(neighbor, &maylive);
                  }
                  break;
               case 4:
                  if (map[nbrrow][nbrcol] == ALIVE) {
                     neighbor.row = nbrrow;
                     neighbor.col = nbrcol;
                     AddList(neighbor, &maydie);
                  }
                  break;
            }  // switch

         }
}


void Vivify(Cell cell)
{// Determine if conditions are right for cell to be brought to life
   if (map[cell.row][cell.col] == DEAD &&
       numNeighbors[cell.row][cell.col] == 3)
         if (cell.row >= 1 && cell.row <= MAXROW &&
             cell.col >= 1 && cell.col <= MAXCOL) {
         	map[cell.row][cell.col] = ALIVE;
         	AddList(cell, &newlive);
         }
}

void SubtractNeighbors(Cell cell)
{// Check number of neighbors around cell that has recently died
 // Update neighbor counts of cells around cell of interest; add to appropriate list
   int nbrrow,              // loop index for row of neighbor loops
       nbrcol;              // column loop index
   Cell neighbor;           // structure form of a neighbor

   for (nbrrow = cell.row-1; nbrrow <=cell.row+1; nbrrow++)
      for (nbrcol = cell.col-1; nbrcol <=cell.col+1; nbrcol++)
         if (nbrrow != cell.row || nbrcol != cell.col) {  //skip cell itself
            numNeighbors[nbrrow][nbrcol]--;
            switch(numNeighbors[nbrrow][nbrcol]) {
               case 8:
                  Error("Impossible case in SubtractNeighbors.");
                  break;
               case 3: 
                  if (map[nbrrow][nbrcol] == DEAD) {
                     neighbor.row = nbrrow;
                     neighbor.col = nbrcol;
                     AddList(neighbor, &maylive);
                  }
                  break;
               case 1:
                  if (map[nbrrow][nbrcol] == ALIVE) {
                     neighbor.row = nbrrow;
                     neighbor.col = nbrcol;
                     AddList(neighbor, &maydie);
                  }
                  break;
            }  // switch

         }
}

void Kill(Cell cell)
{// Determine if conditions are right for cell to be killed
   if (map[cell.row][cell.col] == ALIVE &&
      (numNeighbors[cell.row][cell.col] > 3 ||
       numNeighbors[cell.row][cell.col] < 2))
         if (cell.row >= 1 && cell.row <= MAXROW &&
              cell.col >= 1 && cell.col <= MAXCOL) {
         	map[cell.row][cell.col] = DEAD;
         	AddList(cell, &newdie);
         }
}

void Initialize(Grid map, Gridcount numNeighbors, 
                List *newlive, List *newdie, 
                List *maylive, List *maydie,
                char *fName, int *gens, Cell *c1, Cell *c2)
{
   int row, col;

   CreateList(newlive);
   CreateList(newdie);
   CreateList(maylive);
   CreateList(maydie);

// Obtain the initial configuration
   ReadMap(newlive, map, fName, gens, c1, c2);

// Set all entries in numNeighbors to 0
   for (row=0; row <= MAXROW+1; row++)
      for (col=0; col <= MAXCOL+1; col++)
         numNeighbors[row][col] = 0;

// Put the candidates to live into maylive
   TraverseList(newlive, AddNeighbors);

// Check all living cells to see which may die
   CopyList(maydie, newlive);
   ClearList(newlive);
}
void WriteMap(Grid map, int gc, Cell c1, Cell c2)
{ // Print out the desired section of map after the final simuation generation
   int row, col;
   int i;

   putchar('\n');
   putchar('\n');
   for (i=0; i < (MAXCOL/2-7); i++) putchar(' ');
   printf("GENERATION #%d\n",gc);
   putchar('\n');
   for (row=c1.row; row<=c2.row; row++){
      for (col=c1.col; col<=c2.col; col++)
         if(map[row][col] == ALIVE)
            putchar('*');
         else
            putchar('-');
      putchar('\n');
   }
}

void usage(void)
{
	printf("Usage: gameoflife.exe <file name>\n");
}

void ReadMap(List *newlive, Grid map, char *fName, int *gens, Cell *c1, Cell *c2)
{//Read in the initial configuration of cells, number of generations to simulate,
 //   and the region to be printed after the final generation
   int row, col;
   Cell newcell;
   FILE *fp;
   int good=0, bad=0, dup=0;

   for (row=0; row <= MAXROW+1; row++)
      for (col=0; col <= MAXCOL+1; col++)
         map[row][col] = DEAD;     // Set all cells empty, including the hedge

   if ((fp = fopen(fName, "r")) == NULL) Error("Input file not found");
   printf("Reading input data from %s...",fName); 
   fscanf(fp,"%d %d",&row, &col);
   //Read row,col pairs until 0,0 marker read
   while (row != 0 || col != 0) {
      if (row >= 1 && row <= MAXROW && col >= 1 && col <= MAXCOL) {
         newcell.row = row;
         newcell.col = col;
         if (map[row][col] == DEAD) {
            AddList(newcell, newlive); // no dups
            good++;
         }
         else { 
            dup++;
         }
         map[row][col] = ALIVE;
      }
      else {
         bad++;
      }
      fscanf(fp,"%d %d",&row, &col);
   }
   // Get number of generations to be simulated
   fscanf(fp,"%d", gens);
   // Get region bounds for final print
   fscanf(fp,"%d %d %d %d",&(c1->row), &(c1->col), &(c2->row), &(c2->col));

   printf("Done.\n %d Cells created",good);
   if (bad > 0) printf (", %d Cells out of range",bad);
   if (dup > 0) printf ("and %d Cells duplicated",dup);
   printf("\nNumber of generations to simulate: %d\n", *gens);
   printf("Final print area is %d %d to %d %d\n", c1->row, c1->col, c2->row, c2->col);
   printf("\n");
   fclose(fp);
}
