using System;
using MCSharp;
using System.Reflection;
using System.Threading;
using System.Runtime.Serialization;
using System.Runtime.Remoting.Messaging;

using GpuDotNet.Cuda;
[Serializable()]
public static  class BlackScholes {

const  int OPT_N = 65536  ;

 const  float RISKFREE = 0.02f  ;

 const  float VOLATILITY = 0.30f  ;

 public static  void Main (  String[] args )  {

  if ( ! MCSharp.Session.Init ( args ) ) return;

  Main_MCSharp_Wrapper ( args );

  MCSharp.Session.FinalizeIt();

 }

 private static void Main_MCSharp_Wrapper (  String[] args )  {
   int i ;

  Console.WriteLine( "OPT_N = " + OPT_N   )  ;
 Console.WriteLine( "Initializing data ..."   )  ;
 Console.WriteLine( " ... allocating CPU memory for options."   )  ;
 float[] h_CallResultCPU = new float [ OPT_N   ]    ;

  float[] h_PutResultCPU = new float [ OPT_N   ]    ;

  float[] h_StockPrice = new float [ OPT_N   ]    ;

  float[] h_OptionStrike = new float [ OPT_N   ]    ;

  float[] h_OptionYears = new float [ OPT_N   ]    ;

  float[] d_CallResult = new float [ OPT_N   ]    ;

  float[] d_PutResult = new float [ OPT_N   ]    ;

  Random rnd = new Random ( 5347   )   ;

  for ( i = 0  ; i  < OPT_N ; i++   )
  {
   h_CallResultCPU[ i   ] = 0.0f  ;
 h_PutResultCPU[ i   ] = -1.0f  ;
 d_CallResult[ i   ] = 0.0f  ;
 d_PutResult[ i   ] = -1.0f  ;
 h_StockPrice[ i   ] = RandFloat( 5.0f   , 30.0f   , rnd   )  ;
 h_OptionStrike[ i   ] = RandFloat( 1.0f   , 100.0f   , rnd   )  ;
 h_OptionYears[ i   ] = RandFloat( 0.25f   , 10.0f   , rnd   )  ;
 }

  int GRID_SIZE = 1024  ;

  int BLOCK_SIZE = 256  ;

  Console.WriteLine( "GRID = " + GRID_SIZE + "    BLOCK = " + BLOCK_SIZE   )  ;
 GpuConfig gpuconfig = new GpuConfig ( )   ;

  gpuconfig.SetGridSize( GRID_SIZE   )  ;
 gpuconfig.SetBlockSize( BLOCK_SIZE   )  ;
 DateTime dt1 = DateTime.Now  ;

  gpuconfig.BlackScholesGPU( d_CallResult   , d_PutResult   , h_StockPrice   , h_OptionStrike   , h_OptionYears   , RISKFREE   , VOLATILITY   , OPT_N   )  ;
 DateTime dt2 = DateTime.Now  ;

  Console.WriteLine( "GPU Time = " + ( dt2 - dt1   ).TotalSeconds + " secs."   )  ;
 Console.WriteLine( "Checking the results ..."   )  ;
 Console.WriteLine( " ... running CPU calculations."   )  ;
 dt1 = DateTime.Now  ;
 BlackScholes_gold.BlackScholesCPU( h_CallResultCPU   , h_PutResultCPU   , h_StockPrice   , h_OptionStrike   , h_OptionYears   , RISKFREE   , VOLATILITY   , OPT_N   )  ;
 dt2 = DateTime.Now  ;
 Console.WriteLine( "CPU Time = " + ( dt2 - dt1   ).TotalSeconds + " sec."   )  ;
 Console.WriteLine( "Comparing the results ..."   )  ;
 double delta , _ref , sum_delta , sum_ref , max_delta , L1norm ;

  sum_delta = 0  ;
 sum_ref = 0  ;
 max_delta = 0  ;
 int i_max_delta = 0  ;

  for ( i = 0  ; i  < OPT_N ; i++   )
  {
   _ref = h_CallResultCPU[ i   ]  ;
 delta = Math.Abs( h_CallResultCPU[ i   ] - d_CallResult[ i   ]   )  ;
 if ( delta  > max_delta  )
    {
   max_delta = delta  ;
 i_max_delta = i  ;
 }

  sum_delta += delta  ;
 sum_ref += Math.Abs( _ref   )  ;
 }

  L1norm = sum_delta / sum_ref  ;
 Console.WriteLine( "L1 norm:" + L1norm   )  ;
 Console.WriteLine( "Max absolute error:" + max_delta   )  ;
 }
public static  float RandFloat (  float low,  float high,  Random rnd )   {
   float t = (float)rnd.NextDouble( )  ;

  return ( 1.0f - t   ) * low + t * high  ;
  }
 public delegate void _BlackScholes_BlackScholesGPU_delegate (  float[] d_CallResult,  float[] d_PutResult,  float[] d_StockPrice,  float[] d_OptionStrike,  float[] d_OptionYears,  float Riskfree,  float Volatility,  int optN ) ;

 public static void BlackScholesGPU ( this GpuConfig gpuconfig,   float[] d_CallResult,  float[] d_PutResult,  float[] d_StockPrice,  float[] d_OptionStrike,  float[] d_OptionYears,  float Riskfree,  float Volatility,  int optN )  
 {
  _BlackScholes_BlackScholesGPU_delegate gpu_method_delegate = _BlackScholes_gpu_BlackScholesGPU;
  GpuRuntime.PrepareAndCallGpuMethod ( gpuconfig, gpu_method_delegate, d_CallResult, d_PutResult, d_StockPrice, d_OptionStrike, d_OptionYears, Riskfree, Volatility, optN );
 }

 public static void _BlackScholes_gpu_BlackScholesGPU (  float[] d_CallResult,  float[] d_PutResult,  float[] d_StockPrice,  float[] d_OptionStrike,  float[] d_OptionYears,  float Riskfree,  float Volatility,  int optN ) 
 {
   int tid = BlockSize.X * BlockIndex.X + ThreadIndex.X  ;

  int THREAD_N = BlockSize.X * GridSize.X  ;

  for ( int opt = tid  ;

  opt  < optN ; opt += THREAD_N   )
 BlackScholesBodyGPU( d_CallResult   , d_PutResult   , d_StockPrice[ opt   ]   , d_OptionStrike[ opt   ]   , d_OptionYears[ opt   ]   , Riskfree   , Volatility   , opt   )  ;

  }

public static  void BlackScholesBodyGPU (  float[] d_CallResult,  float[] d_PutResult,  float S,  float X,  float T,  float R,  float V,  int opt )   {
   float sqrtT , expRT ;

  float d1 , d2 , CNDD1 , CNDD2 ;

  sqrtT = GPUMath.sqrtf( T   )  ;
 d1 = ( GPUMath.__logf( S / X   ) + ( R + 0.5f * V * V   ) * T   ) / ( V * sqrtT   )  ;
 d2 = d1 - V * sqrtT  ;
 CNDD1 = cndGPU( d1   )  ;
 CNDD2 = cndGPU( d2   )  ;
 expRT = GPUMath.__expf( -R * T   )  ;
 d_CallResult[ opt   ] = S * CNDD1 - X * expRT * CNDD2  ;
 d_PutResult[ opt   ] = X * expRT * ( 1.0f - CNDD2   ) - S * ( 1.0f - CNDD1   )  ;
 }
public static  float cndGPU (  float d )   {
   const  float A1 = 0.31938153f  ;

  const  float A2 = -0.356563782f  ;

  const  float A3 = 1.781477937f  ;

  const  float A4 = -1.821255978f  ;

  const  float A5 = 1.330274429f  ;

  const  float RSQRT2PI = 0.39894228040143267793994605993438f  ;

  float K = 1.0f / ( 1.0f + 0.2316419f * GPUMath.fabsf( d   )   )  ;

  float cnd = RSQRT2PI * GPUMath.__expf( -0.5f * d * d   ) * ( K * ( A1 + K * ( A2 + K * ( A3 + K * ( A4 + K * A5   )   )   )   )   )  ;

  if ( d  > 0  )
   cnd = 1.0f - cnd  ;

  return cnd  ;
  }
}

