using System;
using MCSharp;
using System.Reflection;
using System.Threading;
using System.Runtime.Serialization;
using System.Runtime.Remoting.Messaging;

using GpuDotNet.Cuda;
[Serializable()]
public static  class gpuMatrixMultDouble {

private const  int BLOCK_SIZE = 16  ;

 static  int WA = ( 3 * BLOCK_SIZE   )  ;

 static  int HA = ( 5 * BLOCK_SIZE   )  ;

 static  int WB = ( 8 * BLOCK_SIZE   )  ;

 static  int HB = WA  ;

 static  int WC = WB  ;

 static  int HC = HA  ;

 public static  void Main (  String[] args )  {

  if ( ! MCSharp.Session.Init ( args ) ) return;

  Main_MCSharp_Wrapper ( args );

  MCSharp.Session.FinalizeIt();

 }

 private static void Main_MCSharp_Wrapper (  String[] args )  {
   if ( args.Length  > 2  )
    {
   HA = Convert.ToInt32( args[ 0   ]   )  ;
 WA = Convert.ToInt32( args[ 1   ]   )  ;
 HB = WA  ;
 WB = Convert.ToInt32( args[ 2   ]   )  ;
 HC = WA  ;
 WC = WB  ;
 }

  double[] A = new double [ HA * WA   ]    ;

  double[] B = new double [ HB * WB   ]    ;

  double[] C = new double [ HC * WC   ]    ;

  for ( int i = 0  ;

  i  < HA * WA ; i++   )
 A[ i   ] = 0.1  ;

  for ( int i = 0  ;

  i  < HB * WB ; i++   )
 B[ i   ] = 0.2  ;

  for ( int i = 0  ;

  i  < HC * WC ; i++   )
 C[ i   ] = 0.0  ;

  GpuConfig gpuconfig = new GpuConfig ( )   ;

  gpuconfig.SetBlockSize( BLOCK_SIZE   , BLOCK_SIZE   )  ;
 gpuconfig.SetGridSize( WC / BLOCK_SIZE   , HC / BLOCK_SIZE   )  ;
 gpuconfig.MatrixMul( C   , A   , B   , WA   , WB   )  ;
 Console.WriteLine( "C[0,0] = " + C[ 0   ]   )  ;
 }
[ StaticArray (BLOCK_SIZE * BLOCK_SIZE    ) ]  private static  Shared1D<double> As ;

 [ StaticArray (BLOCK_SIZE * BLOCK_SIZE    ) ]  private static  Shared1D<double> Bs ;

  public delegate void _gpuMatrixMultDouble_MatrixMul_delegate (  double[] C,  double[] A,  double[] B,  int wA,  int wB ) ;

 public static void MatrixMul ( this GpuConfig gpuconfig,   double[] C,  double[] A,  double[] B,  int wA,  int wB )  
 {
  _gpuMatrixMultDouble_MatrixMul_delegate gpu_method_delegate = _gpuMatrixMultDouble_gpu_MatrixMul;
  GpuRuntime.PrepareAndCallGpuMethod ( gpuconfig, gpu_method_delegate, C, A, B, wA, wB );
 }

 public static void _gpuMatrixMultDouble_gpu_MatrixMul (  double[] C,  double[] A,  double[] B,  int wA,  int wB ) 
 {
   int bx = BlockIndex.X  ;

  int by = BlockIndex.Y  ;

  int tx = ThreadIndex.X  ;

  int ty = ThreadIndex.Y  ;

  int aBegin = wA * BLOCK_SIZE * by  ;

  int aEnd = aBegin + wA - 1  ;

  int aStep = BLOCK_SIZE  ;

  int bBegin = BLOCK_SIZE * bx  ;

  int bStep = BLOCK_SIZE * wB  ;

  double Csub = 0  ;

  for ( int a = aBegin  , b = bBegin  ;

  a  <= aEnd ; a += aStep  , b += bStep   )
  {
   As[ ty * BLOCK_SIZE + tx   ] = A[ a + wA * ty + tx   ]  ;
 Bs[ ty * BLOCK_SIZE + tx   ] = B[ b + wB * ty + tx   ]  ;
 CudaRuntime.SyncThreads( )  ;
 for ( int k = 0  ;

  k  < BLOCK_SIZE ; ++k   )
 Csub += As[ ty * BLOCK_SIZE + k   ] * Bs[ k * BLOCK_SIZE + tx   ]  ;

  CudaRuntime.SyncThreads( )  ;
 }

  int c = wB * BLOCK_SIZE * by + BLOCK_SIZE * bx  ;

  C[ c + wB * ty + tx   ] = Csub  ;
 }

}

