using System;
using MCSharp;
using System.Reflection;
using System.Threading;
using System.Runtime.Serialization;
using System.Runtime.Remoting.Messaging;

using GpuDotNet.Cuda;
[Serializable()]
public static  class gpuSimpleMatrixMult {

private const  int BLOCK_SIZE = 16  ;

 static  int WA = ( 3 * BLOCK_SIZE   )  ;

 static  int HA = ( 5 * BLOCK_SIZE   )  ;

 static  int WB = ( 8 * BLOCK_SIZE   )  ;

 static  int HB = WA  ;

 static  int WC = WB  ;

 static  int HC = HA  ;

 public static  void Main (  String[] args )  {

  if ( ! MCSharp.Session.Init ( args ) ) return;

  Main_MCSharp_Wrapper ( args );

  MCSharp.Session.FinalizeIt();

 }

 private static void Main_MCSharp_Wrapper (  String[] args )  {
   if ( args.Length  > 2  )
    {
   HA = Convert.ToInt32( args[ 0   ]   )  ;
 WA = Convert.ToInt32( args[ 1   ]   )  ;
 HB = WA  ;
 WB = Convert.ToInt32( args[ 2   ]   )  ;
 HC = WA  ;
 WC = WB  ;
 }

  float[] A = new float [ HA * WA   ]    ;

  float[] B = new float [ HB * WB   ]    ;

  float[] C = new float [ HC * WC   ]    ;

  for ( int i = 0  ;

  i  < HA * WA ; i++   )
 A[ i   ] = 0.1f  ;

  for ( int i = 0  ;

  i  < HB * WB ; i++   )
 B[ i   ] = 0.2f  ;

  for ( int i = 0  ;

  i  < HC * WC ; i++   )
 C[ i   ] = 0.0f  ;

  GpuConfig gpuconfig = new GpuConfig ( )   ;

  gpuconfig.SetBlockSize( BLOCK_SIZE   , BLOCK_SIZE   )  ;
 gpuconfig.SetGridSize( WC / BLOCK_SIZE   , HC / BLOCK_SIZE   )  ;
 gpuconfig.MatrixMul( C   , A   , B   , WA   , WB   )  ;
 Console.WriteLine( "C[0,0] = " + C[ 0   ]   )  ;
 }
 public delegate void _gpuSimpleMatrixMult_MatrixMul_delegate (  float[] C,  float[] A,  float[] B,  int wA,  int wB ) ;

 public static void MatrixMul ( this GpuConfig gpuconfig,   float[] C,  float[] A,  float[] B,  int wA,  int wB )  
 {
  _gpuSimpleMatrixMult_MatrixMul_delegate gpu_method_delegate = _gpuSimpleMatrixMult_gpu_MatrixMul;
  GpuRuntime.PrepareAndCallGpuMethod ( gpuconfig, gpu_method_delegate, C, A, B, wA, wB );
 }

 public static void _gpuSimpleMatrixMult_gpu_MatrixMul (  float[] C,  float[] A,  float[] B,  int wA,  int wB ) 
 {
   float Csub = 0  ;

  int row = BlockIndex.Y * BlockSize.Y + ThreadIndex.Y  ;

  int col = BlockIndex.X * BlockSize.X + ThreadIndex.X  ;

  for ( int e = 0  ;

  e  < wA ; ++e   )
 Csub += A[ row * wA + e   ] * B[ e * wB + col   ]  ;

  C[ row * wB + col   ] = Csub  ;
 }

}

