﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using GpuDotNet.Cuda.DriverApi;

namespace GpuDotNet.Cuda.DriverApi
{
	static class DriverUnsafeNativeMethods
	{
		public static void CheckReturnCode(DriverStatusCode rc)
		{
			if (rc != DriverStatusCode.CUDA_SUCCESS)
				throw new CudaException("A CUDA error occurred: " + rc);
		}

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuInit(uint Flags);

		#region Device management

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceGet(out CUdevice deviceOut, int ordinal);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceGetCount(out int count);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceGetName([MarshalAs(UnmanagedType.LPStr)] StringBuilder name, int namelen, CUdevice dev);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceComputeCapability(out int major, out int minor, CUdevice dev);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceTotalMem(out uint bytes, CUdevice dev);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceGetProperties(out CUdevprop prop, CUdevice dev);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuDeviceGetAttribute(out int value, CUdevice_attribute attrib, CUdevice dev);

		#endregion

		#region Context Management

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxCreate(out CUcontext pctx, uint flags, CUdevice dev);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxDestroy(IntPtr ctx);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxAttach(out CUcontextAttachedHandle pctx, uint flags);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxDetach(IntPtr ctx);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxPushCurrent(CUcontext ctx);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxPopCurrent(ref CUcontext pctx);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxGetDevice(out CUdevice device);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuCtxSynchronize();

		#endregion

		#region Module Management

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleLoad(out CUmodule module, [MarshalAs(UnmanagedType.LPStr)] string fname);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleLoadData(out CUmodule module, [MarshalAs(UnmanagedType.LPStr)] string cubinImage);
		[DllImport("libcuda")]
		[Obsolete("Not obsolete, but don't know what the pointer type is.")]
		public extern static DriverStatusCode cuModuleLoadFatBinary(ref CUmodule module, IntPtr fatbin);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleUnload(CUmodule hmod);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleGetFunction(out CUfunction hfunc, CUmodule hmod, [MarshalAs(UnmanagedType.LPStr)] string name);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleGetGlobal(out CUdevice dptr, out uint bytes, CUmodule hmod, [MarshalAs(UnmanagedType.LPStr)] string name);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuModuleGetTexRef(out CUtexref pTexRef, CUmodule hmod, [MarshalAs(UnmanagedType.LPStr)] string name);

		#endregion

		#region Memory Management

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuMemGetInfo(out uint free, out uint total);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuMemAlloc(out CUdeviceptr dptr, uint bytesize);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuMemFree(CUdeviceptr dptr);

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuMemcpyHtoD(CUdeviceptr dstDevice, IntPtr src, uint ByteCount);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuMemcpyDtoH(IntPtr dstHost, CUdeviceptr srcDevice, uint ByteCount);

		#endregion

		#region Execution Control

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuParamSetSize(CUfunction hfunc, uint numbytes);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuParamSeti(CUfunction hfunc, int offset, uint value);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuParamSetf(CUfunction hfunc, int offset, float value);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuParamSetv(CUfunction hfunc, int offset, IntPtr ptr, uint numbytes);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuParamSetTexRef(CUfunction hfunc, int texunit, CUtexref hTexRef);

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuLaunch(CUfunction f );
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuLaunchGrid (CUfunction f, int grid_width, int grid_height);
//		[DllImport("nvcuda")]
//		public extern static DriverStatusCode cuLaunchGridAsync( CUfunction f, int grid_width, int grid_height, CUstream hStream );

		#endregion


		#region Function Management

		[DllImport("libcuda")]
		public extern static DriverStatusCode cuFuncSetBlockShape (CUfunction hfunc, int x, int y, int z);
		[DllImport("libcuda")]
		public extern static DriverStatusCode cuFuncSetSharedSize (CUfunction hfunc, uint bytes);

		#endregion
	}
}
