//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.Runtime.Serialization;

namespace MCSharp {
 [Serializable()]
 public class Agent : ISerializable {
  public object Obj;
  public string AssemblyName;
  public string ClassName;
  public string MethodName;
  public int ObjectNumber;
  public object[] Parameters;

  public Agent( channel c, object[] parameters ) {
   this.Obj = null;
   this.MethodName = c.MethodName;
   this.ObjectNumber = c.ObjectNumber;
   this.Parameters = parameters;
  }

  public Agent(handler handl, object[] parameters)  { //   MC# 2.0
   this.Obj = null;
   this.MethodName = handl.MethodName;
   this.ObjectNumber = handl.ObjectNumber;
   this.Parameters = parameters;
  }

  public Agent( object obj, string methodName, object[] parameters ) {
   this.Obj = obj;
   this.MethodName = methodName;
   this.Parameters = parameters;
   this.AssemblyName = obj.GetType().Module.Name;
   this.ClassName = obj.GetType().Name;
  }

  public Agent( SerializationInfo info, StreamingContext context ) {
   Obj = info.GetValue( "0", typeof( object ) );
   AssemblyName = (string) info.GetValue( "1", typeof( string ) );
   ClassName = (string) info.GetValue( "2", typeof( string ) );
   MethodName = (string) info.GetValue( "3", typeof( string ) );
   ObjectNumber = (int) info.GetValue( "4", typeof( int ) );
   Parameters = (object[]) info.GetValue( "5", typeof( object[] ) );
  }

  public void GetObjectData( SerializationInfo info, StreamingContext context ) {
   info.AddValue( "0", Obj );
   info.AddValue( "1", AssemblyName );
   info.AddValue( "2", ClassName );
   info.AddValue( "3", MethodName );
   info.AddValue( "4", ObjectNumber );
   info.AddValue( "5", Parameters );
  }

  public override string ToString() {
   string result = "(Agent)" +
    "\n  assembly: " + AssemblyName +
    "\n  class name: " + ClassName +
    "\n  method name: " + MethodName +
    "\n  object number: " + ObjectNumber +
    "\n  number of parameters: " + Parameters.Length;
    for ( int i = 0; i < Parameters.Length; i++ ) 
     result += "\n  param [ " + i + " ] = {\n" + Parameters [ i ].ToString() + "\n  }\n";
    result += "\n  object: " + Obj;
    return result;
  }
 }
}
