//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System.Collections;
using System.Net.Sockets;
using System.Threading;

namespace MCSharp {
 public class ApplicationConfig {
  public ArrayList Nodes = new ArrayList();
  public int NumberOfProcessors = 0;
  public int NumberOfLoadedNodes = 0;
  private int CurrentFreeNode = 0;
  public string ClientHost = "";
  public int ClientPort = 0;
  public string CommandLine = "";
  public string CurrentDirectory = "";
  public Socket ClientSocket;
  public bool OneNodeInitiated = false;
  public bool ShowStatistics = false;
  public Hashtable Statistics = new Hashtable();
  public Hashtable AssignedNodes = new Hashtable();
 
  public ApplicationConfig( int numberOfProcessors ) {
   this.NumberOfProcessors = numberOfProcessors;
  }
 
  public Node GetNextFreeNode {
   get {
    Node node = null;
    lock ( Statistics ) {
     CurrentFreeNode++;
     if ( CurrentFreeNode >= NumberOfLoadedNodes ) CurrentFreeNode = 0;
     node = (Node) this.Nodes [CurrentFreeNode];
     if ( Statistics [node] == null ) Statistics [node] = 1;
     else
      Statistics [node] = (int) Statistics [node] + 1;
    }
    return node;
   }
  }

  public void AddNode( Node node ) {
   Nodes.Add( node );
   NumberOfLoadedNodes++;
   CurrentFreeNode = NumberOfLoadedNodes - 1;
  }
  
  public override string ToString() {
   return "(ApplicationConfig)" + 
    "\n  Command: " + CommandLine +
    "\n  Number of processors: " + NumberOfProcessors +
    "\n  Show Statistics: " + ShowStatistics + 
    "\n  Client: (" + ClientHost + ":" + ClientPort + ")";
  }
 }
}
