//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.Threading;

namespace MCSharp {

 public class CommWorld {
  private static int _rank = 0;
  public static int Rank {
   get {
    while ( !_initialized ) {
     Thread.Sleep( 10 );
    }
    return _rank;
   }
  }
  
  public static int _size = 0;
  public static int Size {
   get {
    while ( !_initialized ) {
     Thread.Sleep( 10 );
    }
    return _size;
   }
  }
  
  private static bool _initialized;
  public static bool Initialized {
   get {
    return _initialized;
   }
  }

  public static Node CurrentNode {
   get {
    while ( !_initialized ) {
     Thread.Sleep( 10 );
    }
    return new Node( Communicator.Host, 
     Communicator.Port );
   }
  }

  private static Node[] nodes = null;
  public static Node[] Nodes {
   get {
    while ( !_initialized ) {
     Thread.Sleep( 10 );
    }
    return nodes;
   }
  }
  
  internal static void Initialize( string[] communicators ) {
   _size = communicators.Length;
   nodes = new Node [_size];
   for ( int i = 0; i < _size; i++ ) {
    string[] tokens = communicators [i].Split( ' ' );
    nodes [i] = new Node( tokens [0], int.Parse( tokens [1] ) );
    if ( nodes [i].Host.Equals( Communicator.Host ) &&
         nodes [i].Port == Communicator.Port ) {
     _rank = i;
    }
   }
   _initialized = true;
  }
  
  public static string Info() {
   while ( !_initialized ) {
    Thread.Sleep( 10 );
   }
   string ret = "Current rank is " + Rank +
    "\nThere are " + Size + " nodes in cluster\n";
   for ( int i = 0; i < Size; i++ )
    ret += "  " + i + ". " + nodes [i].Host + ":" + nodes [i].Port + "\n";
   return ret;
  }
 }
}
