//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections;

namespace MCSharp {
 public class Communicator {
  public static bool IsOnClientNode = true;  // Initiated in Session.Init
  public static bool IsOnClusterNode = false;  // Initiated in Session.Init
  public static string RMHost = "";
  public static int RMPort = 0;
  
  //public static bool initiated = false;

  public static string Host = "";
  public static int Port = 0;

  private static Socket serverSocket;
  
  public static Hashtable BDChannels = new Hashtable( 100 );
  public static Hashtable Activities = new Hashtable( 100 );
  
  private Communicator() {}
  
  public static void Init( string hostName ) {
       
   if ( Session.Distributed ) {
        
    serverSocket = TCP.CreateServerSocket( 10000, 30000, out Port );
        
    Thread communicatorThread = new Thread( new ThreadStart( Launch ) );
    communicatorThread.Start();
        
    while ( ! communicatorThread.IsAlive )
     Thread.Sleep ( 50 );
             
   }
   
   Host = hostName;
      
  }

  private static bool NeedToContinue = true;
    
  public static void Launch() {

if ( Globals.IsDebug ) {
   Console.WriteLine( "Started Communicator on {0}:{1}", Host, Port );
}
   NeedToContinue = true;

   while ( NeedToContinue ) {
    Socket client = serverSocket.Accept();
    string command = TCP.ReadLine( client );
        
    if ( command.Equals ( "" ) )
     Console.WriteLine ( "Communicator - command is null !!!" );
    
    if ( command.StartsWith( "movablecall" ) || 
         command.StartsWith( "handlerGet")       ) {
     
     Thread t = new Thread( new ThreadStart( new CommunicatorHandler( client, command ).Handle ) );
     t.Start();
     
    }
    else if ( command == "stopcommunicator" ) {
     NeedToContinue = false;
     client.Close();
    }
    else {
     new CommunicatorHandler( client, command ).Handle();
    }
   }
   if ( Session.Distributed ) {
    serverSocket.Close();
   }
  }
  
  public static void FinalizeIt() {
   if ( Session.Distributed ) {
    TCP.SendMessage( Host, Port, "stopcommunicator\n" );
   }
  }
 }
}
