//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections;

namespace MCSharp {
 public class CommunicatorHandler {
 
  public static object deserializeLock = new object ();
  
  Socket client;
  string command;

  public CommunicatorHandler( Socket client, string command ) {
   this.client = client;
   this.command = command;
  }

  public void Handle() {
   if ( client == null ) return;

   if ( command == null || command.Length == 0 ) return;
if ( Globals.IsDebug ) {
   Monitor.Enter( MCException.lockString );
    Console.WriteLine( Communicator.Host + ":" + Communicator.Port + " <-: " + command );
   Monitor.Exit( MCException.lockString );
}
   
   string[] tokens = command.Split( ' ' );
   if ( tokens [ 0 ].Equals( "applicationinitiated" ) ) {
    ////////////////////////////////////////////////////////////////////////////
    // Message format: "applicationinitiated\n
    // numberOfProcessors\n
    // clientHost clientPort\n
    // hostname_1 portname_1\n
    // ...
    // hostname_n portname_n\n
    ////////////////////////////////////////////////////////////////////////////
    int numberOfProcessors = int.Parse( TCP.ReadLine( client ) );
    string[] wns = new string [numberOfProcessors + 1];
    for ( int i = 0; i < numberOfProcessors + 1; i++ )
     wns [i] = TCP.ReadLine( client );
    
    CommWorld.Initialize( wns );
    Session.Initiated = true;
    Session.Finalized = false;
    client.Close();
   }
   else if ( tokens [ 0 ].Equals( "sessioninitiated" ) ) {
    ////////////////////////////////////////////////////////////////////////////
    // Message format: "sessioninitiated\n"
    // numberOfProcessors\n
    // clientHost clientPort\n
    // hostname_1 portname_1\n
    // ...
    // hostname_n portname_n\n
    ////////////////////////////////////////////////////////////////////////////
    int numberOfWns = int.Parse( TCP.ReadLine( client ) );
    
        
    string[] lines = new string [numberOfWns + 1];
    for ( int j = 0; j < numberOfWns + 1; j++ ) {
     lines [j] = TCP.ReadLine( client );
     
    }
    CommWorld.Initialize( lines );
    
    client.Close();
   }
   else if ( tokens [ 0 ].Equals( "stop" ) ) {
    ////////////////////////////////////////////////////////////////////////////
    // Message format: "stop\n"
    ////////////////////////////////////////////////////////////////////////////
    Communicator.FinalizeIt();
    CommExec.FinalizeIt();
    TCP.SendMessage( client, MCStatistics.CurrentStatistics.ToString2() );
    client.Close();
    System.Environment.Exit( 0 );
   }
   else if ( tokens [ 0 ].Equals( "movablecall" ) ) {
    ////////////////////////////////////////////////////////////////////////////
    // Message format: "movablecall\nAgent"
    ////////////////////////////////////////////////////////////////////////////
            
    int bufLength = int.Parse( TCP.ReadLine( client ) );
    MemoryStream ms = new MemoryStream( bufLength );
    TCP.ReadDataToStream( client, ms );
    client.Close();
    MCStatistics.CurrentStatistics.totalSizeOfMovableCallsMessagesAcrossNetwork += ms.Length;
    MCStatistics.CurrentStatistics.numberOfMovableCallsAcrossNetwork++;
    ms.Seek( 0, SeekOrigin.Begin );
    Monitor.Enter ( deserializeLock );
    Agent agent = (MCSharp.Agent) Serialization.Deserialize( ms );
    Monitor.Exit ( deserializeLock );
        
    new MovableMethodHandler( agent ).StartInNewThread();
   }
   else if (tokens[0].Equals("handlerGet"))   
   {
       ////////////////////////////////////////////////////////////////////////////
       // Message format: "handlerGet\nAgent"                                    //
       ////////////////////////////////////////////////////////////////////////////
       int bufLength = int.Parse(TCP.ReadLine(client));
       MemoryStream ms = new MemoryStream(bufLength);
       TCP.ReadDataToStreamWithLength(client, ms, bufLength);
       
       // Add the statistics here
       ms.Seek(0, SeekOrigin.Begin);
       Agent agent = (MCSharp.Agent)Serialization.Deserialize(ms);
       
       new HandlerGetHandler(client, agent).Start();
       client.Close();
   }
   
   else if (tokens[0].Equals("channelmessage"))
   {
       ////////////////////////////////////////////////////////////////////////////
       // Message format: "channelmessage\nAgent"
       ////////////////////////////////////////////////////////////////////////////
       CommExec.EnqueueChannelMessage(client);
   }
   else if (tokens[0].Equals("stopruntime"))
   {
       ////////////////////////////////////////////////////////////////////////////
       // Message format: "stopruntime\n"
       ////////////////////////////////////////////////////////////////////////////
       System.Environment.Exit(0);
   }
   else if (tokens[0].Equals("runtimestopped"))
   {
       ////////////////////////////////////////////////////////////////////////////
       // Message format: "runtimestopped\n"
       ////////////////////////////////////////////////////////////////////////////
       Console.WriteLine(
        "-----------------------------------------------------------------------------\n" +
        "Error! MC# Runtime has been stopped!\n" +
        "-----------------------------------------------------------------------------"
       );
       System.Environment.Exit(0);
   }
   else if (tokens[0].Equals("exception"))
   {
       ////////////////////////////////////////////////////////////////////////////
       // Message format: "exception\n
       ////////////////////////////////////////////////////////////////////////////
   }
   else
   {
       Console.WriteLine("unknown command: \"" + tokens[0] + "\"");
   }
  }
 }
}
