//
// Copyright (C) 2012 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//                      
using System;
using GpuDotNet.Cuda;

public class GpuRuntime   {

 public static void PrepareAndCallGpuMethod ( GpuConfig gpuconfig, Delegate gpu_method_delegate, params object[] args ) {

  CudaKernel kernel = CudaKernel.Create ( gpu_method_delegate, gpuconfig.GetDeviceNumber() );

  object[] device_args = new object [ args.Length ];

  int  i = 0;
  foreach ( object arg in args ) {

   Type type = arg.GetType();
      
   if ( type.IsArray ) {
   
    Type element_type = type.GetElementType();
    
    if ( element_type.Equals ( typeof (int)    )  )  {
     GlobalMemory<int>  device_arg = kernel.Context.AllocateLinear<int> ( ((Array)arg).Length );
     kernel.Context.CopyHostToDevice ( (int[])arg, 0, device_arg, 0, ((int[])arg).Length );
     device_args [ i ] = device_arg;
    }
    else
     if ( element_type.Equals ( typeof (float)    )  )  {
      GlobalMemory<float>  device_arg = kernel.Context.AllocateLinear<float> ( ((Array)arg).Length );
      kernel.Context.CopyHostToDevice ( (float[])arg, 0, device_arg, 0, ((float[])arg).Length );
      device_args [ i ] = device_arg;
     }
     else
      if ( element_type.Equals ( typeof (double)    )  )  {
       GlobalMemory<double>  device_arg = kernel.Context.AllocateLinear<double> ( ((Array)arg).Length );
       kernel.Context.CopyHostToDevice ( (double[])arg, 0, device_arg, 0, ((double[])arg).Length );
       device_args [ i ] = device_arg;
      }
      else
       Console.WriteLine ( "Error: gpu methods support int, float and doubele arrays only." );
   }
   else
    device_args [ i ] = arg;

   i++;
  }

  //    Set the grid and block sizes

  kernel.SetGridSize  ( gpuconfig.GetGridSizeX(),  gpuconfig.GetGridSizeY() );
  kernel.SetBlockSize ( gpuconfig.GetBlockSizeX(), gpuconfig.GetBlockSizeY(), gpuconfig.GetBlockSizeZ() );

  //    Run the kernel

  kernel.ExecuteUntyped ( device_args );
  kernel.Context.Synchronize();

  //    Copy the device results to the main memory

  i = 0;
  foreach ( object arg in args ) {

   Type type = arg.GetType();
      
   if ( type.IsArray )  {
   
    Type element_type = type.GetElementType();
    
    if ( element_type.Equals ( typeof (int)    )  )
     kernel.Context.CopyDeviceToHost ( (GlobalMemory<int>)device_args[i], 0, (int[])arg, 0, ((int[])arg).Length );
    else
     if ( element_type.Equals ( typeof (float)    )  )
      kernel.Context.CopyDeviceToHost ( (GlobalMemory<float>)device_args[i], 0, (float[])arg, 0, ((float[])arg).Length );                                     
     else
      if ( element_type.Equals ( typeof (double)    )  )
       kernel.Context.CopyDeviceToHost ( (GlobalMemory<double>)device_args[i], 0, (double[])arg, 0, ((double[])arg).Length );
      else
       Console.WriteLine ( "Error: gpu methods support int, float and doubele arrays only." );
   }
      
   i++;

  }

 }

}
