//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.Collections;
using System.Threading;
using System.Runtime.Serialization;
using System.IO;
using System.Reflection;

namespace MCSharp   {
 
 [Serializable()]

 public class handler: ISerializable {

  public String Host = ""; // Original host
  public int    Port = 0;  // Original port

  public int    ObjectNumber = 0;  // Number of the object to which this handler belongs
  public String MethodName   = ""; // Name of the method to get a value through the handler
  
  [NonSerialized] public bool isCreatedOnThisMachine;

  //       Constructor

  public handler ( object o, int objectNumber, String methodName )  {

   this.Host = Communicator.Host;
   this.Port = Communicator.Port;

   this.ObjectNumber = objectNumber;
   this.MethodName   = methodName;
   
   this.isCreatedOnThisMachine = true;

  }

  //        Method to serialize given handler

  public void GetObjectData ( SerializationInfo info, StreamingContext context ) {

   info.AddValue ( "1", Host );
   info.AddValue ( "2", Port );
   info.AddValue ( "3", ObjectNumber );
   info.AddValue ( "4", MethodName );
   
  }

  //       Deserialization constructor

  public handler ( SerializationInfo info, StreamingContext context )   {

   Host         = (String) info.GetValue ( "1", typeof ( String ) );
   Port         = (int)    info.GetValue ( "2", typeof ( int )    );
   ObjectNumber = (int)    info.GetValue ( "3", typeof ( int )    );
   MethodName   = (String) info.GetValue ( "4", typeof ( String ) );
         
   isCreatedOnThisMachine = false;

  }
  
  //    Get a value

     public object Get(params object[] args)
     {
       
         if (!Session.Distributed ||
              (Communicator.Host.Equals(Host) &&
                Communicator.Port == Port)
            )
         {
             object obj = CommExec.Objects[ObjectNumber];
             Type t = obj.GetType();
	     MethodInfo minfo = t.GetMethod(MethodName);
	     
	     return ( minfo.Invoke(obj, args  ) );
         }
         else
             return ( TCP.getRemoteValue(this, args) );

     }
   
 }

}
