//
// Copyright (C) 2012 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//
//    

using System.Threading;
using System;
using System.IO;
using System.Reflection;
using System.Net;
using System.Net.Sockets;

namespace MCSharp {
 class HandlerGetHandler {

  private MCSharp.Agent agent;
  private Socket        client;

  
  private HandlerGetHandler() {}

  public HandlerGetHandler( Socket client, Agent agent ) {
   this.client = client;
   this.agent  = agent;
  }
  
  public void Start() {
   
   object resultObject = null;
   
   try {

    object    obj = CommExec.Objects[agent.ObjectNumber];
    Type        t = obj.GetType();
    MethodInfo mi = t.GetMethod( agent.MethodName );
    
    resultObject = mi.Invoke( obj, agent.Parameters ); 
       
   }
   catch ( Exception e ) {
    Console.WriteLine ( "EXCEPTION in HandlerGetHandler: " + e.Message );
    if ( !Session.Finalized ) {
     MCException.WriteException( "MMH0001", "", e );
     MCException.WriteException( "MMH0001-inner", "", e.InnerException );
    }
   }   
   
   byte[]  b;
   int ms_size = 16384;
   
   if ( resultObject == null) 
    b = new byte [ 0 ];
   else  {    
    
    if ( resultObject.GetType().IsArray )
     ms_size += TCP.GetArraySize ( resultObject );     
   
    MemoryStream ms = new MemoryStream(ms_size);
    Serialization.Serialize(ms, resultObject);
                                
    b = ms.ToArray();
    
    
   }
   
   // Sending length of the buffer
   TCP.SendMessage(client, b.Length + "\n");
   TCP.SendMessage(client, b, b.Length);
      
      
  }  //   The end of Start method
   
 }   //   The end of HandlerGetHandler class

}
