//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;

namespace MCSharp {
 public class MCStatistics {
  public static MCStatistics CurrentStatistics;
  public long numberOfMovableCalls = 0;
  public long numberOfChannelMessages = 0;
  public long numberOfMovableCallsAcrossNetwork = 0;
  public long numberOfChannelMessagesAcrossNetwork = 0;
  public long totalSizeOfMovableCallsMessagesAcrossNetwork = 0;
  public long totalSizeOfChannelMessagesAcrossNetwork = 0;
  public long totalTimeOfMovableCallsSerialization = 0;
  public long totalTimeOfChannelMessagesSerialization = 0;
  public long numberOfBytesSentAcrossNetwork = 0;
  public long totalTimeOfTransportingMessages = 0;
  public TimeSpan SessionInitializationTime;
  
  public MCStatistics() {
  }

  public void Add( MCStatistics stats ) {
   numberOfMovableCalls += stats.numberOfMovableCalls;
   numberOfChannelMessages += stats.numberOfChannelMessages;
   numberOfMovableCallsAcrossNetwork += stats.numberOfMovableCallsAcrossNetwork;
   numberOfChannelMessagesAcrossNetwork += stats.numberOfChannelMessagesAcrossNetwork;
   totalSizeOfMovableCallsMessagesAcrossNetwork += stats.totalSizeOfMovableCallsMessagesAcrossNetwork;
   totalSizeOfChannelMessagesAcrossNetwork += stats.totalSizeOfChannelMessagesAcrossNetwork;
   totalTimeOfMovableCallsSerialization += stats.totalTimeOfMovableCallsSerialization;
   totalTimeOfChannelMessagesSerialization += stats.totalTimeOfChannelMessagesSerialization;
   numberOfBytesSentAcrossNetwork += stats.numberOfBytesSentAcrossNetwork;
   totalTimeOfTransportingMessages += stats.totalTimeOfTransportingMessages;
  }
  
  public MCStatistics( string statistics ) {
   string[] lines = statistics.Split( '\n' );
   numberOfMovableCalls = long.Parse( lines [0] );
   numberOfChannelMessages = long.Parse( lines [1] );
   numberOfMovableCallsAcrossNetwork = long.Parse( lines [2] );
   numberOfChannelMessagesAcrossNetwork = long.Parse( lines [3] );
   totalSizeOfMovableCallsMessagesAcrossNetwork = long.Parse( lines [4] );
   totalSizeOfChannelMessagesAcrossNetwork = long.Parse( lines [5] );
   totalTimeOfMovableCallsSerialization = long.Parse( lines [6] );
   totalTimeOfChannelMessagesSerialization = long.Parse( lines [7] );
   numberOfBytesSentAcrossNetwork = long.Parse( lines [8] );
   totalTimeOfTransportingMessages = long.Parse( lines [9] );
  }

  public override string ToString() {
   
   return 
    "Number of movable calls: " + numberOfMovableCalls + "\n" +
    "Number of channel messages: " + numberOfChannelMessages + "\n" +
    "Number of movable calls (across the network): " + numberOfMovableCallsAcrossNetwork + "\n" +
    "Number of channel messages (across the network): " + numberOfChannelMessagesAcrossNetwork + "\n" +
    "Total size of movable calls (across the network): " + totalSizeOfMovableCallsMessagesAcrossNetwork + " bytes\n" +
    "Total size of channel messages (across the network): " + totalSizeOfChannelMessagesAcrossNetwork + " bytes\n" +
    "Total time of movable calls serialization: " + new TimeSpan( totalTimeOfMovableCallsSerialization ) + "\n" +
    "Total time of channel messages serialization: " + new TimeSpan( totalTimeOfChannelMessagesSerialization ) + "\n" +
    "Total size of transported messages: " + numberOfBytesSentAcrossNetwork + " bytes\n" +
    "Total time of transporting messages: " + new TimeSpan( totalTimeOfTransportingMessages ) + "\n";
  }

  public string ToString2() {
   return 
    numberOfMovableCalls + "\n" +
    numberOfChannelMessages + "\n" +
    numberOfMovableCallsAcrossNetwork + "\n" +
    numberOfChannelMessagesAcrossNetwork + "\n" +
    totalSizeOfMovableCallsMessagesAcrossNetwork + "\n" +
    totalSizeOfChannelMessagesAcrossNetwork + "\n" +
    totalTimeOfMovableCallsSerialization + "\n" +
    totalTimeOfChannelMessagesSerialization + "\n" +
    numberOfBytesSentAcrossNetwork + "\n" +
    totalTimeOfTransportingMessages + "\n";
  }

 }
}
