//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections;
using System.Diagnostics;
using MCSharp;

class ResourceManager {
 public static int Port = 8888;
 public static string Host = "";
 public static int WNPort = 9999;
 
 public static ArrayList Nodes = new ArrayList();
 public static Hashtable Applications = new Hashtable( 10 );

 public static int CurrentApplication = 0;

 // resman fileWithNodes
 public static void Main( string[] args ) {
  IPAddress[] addressList = Dns.GetHostByName( Dns.GetHostName() ).AddressList;
  ResourceManager.Host = addressList [0].ToString();

  string home = Environment.GetEnvironmentVariable( "HOME" );
  try {
   FileStream fs = new FileStream( home + Path.DirectorySeparatorChar + ".mcsharp" +
    Path.DirectorySeparatorChar + ".rmport", FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   ResourceManager.Port = int.Parse( sr.ReadLine() );
   fs.Close();
  }
  catch {
   Console.WriteLine( "File ~/.mcsharp/.rmport wasn't found... Setting RM port to 8888!" );
   ResourceManager.Port = 8888;
  }

  try {
   FileStream fs = new FileStream( home + Path.DirectorySeparatorChar + ".mcsharp" +
    Path.DirectorySeparatorChar + ".wnport", FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   ResourceManager.WNPort = int.Parse( sr.ReadLine() );
   fs.Close();
  }
  catch {
   Console.WriteLine( "File ~/.mcsharp/.wnport wasn't found... Setting WN port to 9999!" );
   ResourceManager.WNPort = 9999;
  }

  string curDir = Environment.CurrentDirectory;
  if ( !curDir.EndsWith( "/" ) ) curDir += "/";
      
  string mcsharpPath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );
  if ( !mcsharpPath.EndsWith( "/" ) ) mcsharpPath += "/";
	  
  string fileName = mcsharpPath + "bin/nodes";
  if ( args.Length != 0 ) {
   fileName = args [0];
   if ( !fileName.StartsWith( "/" ) ) {
    fileName = curDir + fileName;
   }
  }

  try {
   IPEndPoint ipep = new IPEndPoint( IPAddress.Any, ResourceManager.Port );
   Socket serverSocket = new Socket( AddressFamily.InterNetwork, 
    SocketType.Stream, ProtocolType.Tcp );
   serverSocket.Bind( ipep );
   serverSocket.Listen( 1 );
   serverSocket.Close();
  }
  catch ( Exception e ) {
   Console.WriteLine( "MC# Runtime Error : Cluster has already been loaded or port " +
    ResourceManager.Port + " is already in use!" );
   Console.WriteLine( " Run 'mcshalt' and then try to launch 'mcsboot'." );
   Console.WriteLine( " If it doesn't help then change the Resource Manager's" );
   Console.WriteLine( " port number in file '" + home + Path.DirectorySeparatorChar + 
   ".mcsharp" + Path.DirectorySeparatorChar + ".rmport" + "'" );
   return;
  }

  Start();
 }

 public static void Start() {
  IPEndPoint ipep = new IPEndPoint( IPAddress.Any, ResourceManager.Port );
  Socket serverSocket = new Socket( AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp );
  serverSocket.Bind( ipep );
  serverSocket.Listen( 100 );

  Console.WriteLine( "Started Resource manager 2.0 on {0}:{1}", Host, Port );

  while ( true ) {
   try {
    Socket client = serverSocket.Accept();
    if ( client == null )
     Console.WriteLine ( "Socket = null after Accept ..." );
    ResourceManagerHandler.Handle( client );
   }
   catch ( Exception e ) {
    MCException.WriteException( "RM0002", "", e );
   }
  }
 }
}

public class ResourceManagerHandler {

 public static void Handle( Socket client ) {
  
  string command = TCP.ReadLine( client );
    
  if ( Globals.IsDebug ) {
   Console.WriteLine( "<-: " + command );
  }

  string[] tokens = command.Split( ' ' );
  if ( tokens [ 0 ].Equals( "wns" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "wns numberOfWns\n
   // wn1\n
   // ...
   // wnK\n"
   ////////////////////////////////////////////////////////////////////////////
   int numberOfWns = int.Parse( tokens [1] );
   for ( int i = 0; i < numberOfWns; i++ ) {
    string wn = TCP.ReadLine( client );
    ResourceManager.Nodes.Add( wn );
   }
   client.Close();
  }
  else if ( tokens [ 0 ].Equals( "init" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "init numberOfProcessors clientHost clientPort\n
   // commandLine\n"
   ////////////////////////////////////////////////////////////////////////////
   InitApplication( tokens, client );
  }
  else if ( tokens [0].Equals( "initiated" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "initiated ApplicationGUID host port\n
   ////////////////////////////////////////////////////////////////////////////
   string guid = tokens [1];
   string host = tokens [2];
   int port = int.Parse( tokens [3] );
   if ( ResourceManager.Applications [guid] != null ) {
    ApplicationConfig ac = (ApplicationConfig) ResourceManager.Applications [guid];
    
    //IPEndPoint client_ipep = (IPEndPoint)ac.ClientSocket.RemoteEndPoint;
    //Console.WriteLine ( "host=" + client_ipep.Address.ToString() );
    //Console.WriteLine ( "port=" + client_ipep.Port.ToString()    );
    //Console.WriteLine ( "connected=" + ac.ClientSocket.Connected );
    //Console.WriteLine ( "poll=" + ac.ClientSocket.Poll ( -1, SelectMode.SelectWrite ) );
    
    ac.AddNode( new Node( host, port ) );
            
    if ( !ac.OneNodeInitiated ) {
     ac.OneNodeInitiated = true;
     TCP.SendMessage( ac.ClientSocket, "initstarted\n" + guid + "\n" );
          
     ac.ClientSocket.Close();
    }
    
    if ( ac.NumberOfLoadedNodes == ac.NumberOfProcessors ) {
     string toSend = ac.NumberOfLoadedNodes + "\n" + ac.ClientHost + " " + ac.ClientPort + "\n";
     foreach ( Node n in ac.Nodes )
      toSend += n.Host + " " + n.Port + "\n";

     foreach ( Node n in ac.Nodes )
      TCP.SendMessage( n.Host, n.Port, "applicationinitiated\n" + toSend );

     TCP.SendMessage( ac.ClientHost, ac.ClientPort, "sessioninitiated\n" + toSend );
    }
    client.Close();
   }
  }
  else if ( tokens [0].Equals( "getfreenode" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "getfreenode ApplicationGUID\n"
   ////////////////////////////////////////////////////////////////////////////
   string guid = tokens [1];
   object o = ResourceManager.Applications [guid];
   if ( o == null ) return;
   ApplicationConfig ac = (ApplicationConfig) o;
   Node node = ac.GetNextFreeNode;   
if ( Globals.IsDebug ) {
 Console.WriteLine( "->: " + node.Host + "\n" + node.Port + "\n" );
}
   TCP.SendMessage( client, node.Host + "\n" + node.Port + "\n" );
  }
  else if ( tokens [0].Equals( "stop" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "stop ApplicationGUID\n"
   ////////////////////////////////////////////////////////////////////////////
   string guid = tokens [1];
   object o = ResourceManager.Applications [guid];
   if ( o == null ) return;
   ApplicationConfig ac = (ApplicationConfig) o;
   MCStatistics statistics = new MCStatistics();
   for ( int j = 0; j < ac.Nodes.Count; j++ ) {
    Node n = (Node)ac.Nodes [j];
    Socket client2 = TCP.SendMessage( n.Host, n.Port, "stop\n" );
    string stats = "";
    for ( int i = 0; i < 10; i++ ) {
     stats += TCP.ReadLine( client2 ) + "\n";
    }
    statistics.Add( new MCStatistics( stats ) );    
    client2.Close();
   }
   StringBuilder movableStats = new StringBuilder( "" );
   if ( ac.ShowStatistics ) {
    Monitor.Enter( ac.Statistics );
    Hashtable mmStats = new Hashtable();
    foreach ( DictionaryEntry de in ac.Statistics ) {
     string key = ((Node)de.Key).Host;     
     if ( mmStats [key] == null ) {
      mmStats [key] = (int)de.Value;
     }
     else {
      mmStats [key] = (int) mmStats [key] + (int)de.Value;
     }
     
    }
    movableStats.Append( ResourceManager.Nodes.Count + "\n" );
    movableStats.Append( ac.AssignedNodes.Count + "\n" );
    foreach ( DictionaryEntry de in ac.AssignedNodes ) {
     movableStats.Append( de.Value + " x " + de.Key );
     movableStats.Append( "\n" );
     if ( mmStats [de.Key] != null ) 
      movableStats.Append( mmStats [de.Key] );
     else
      movableStats.Append( "0" );
     movableStats.Append( "\n" );
    }
    Monitor.Exit( ac.Statistics );
   }
   ResourceManager.Applications.Remove( guid );
if ( Globals.IsDebug ) {
 Console.WriteLine( "-> client:" + statistics.ToString2() );
}
   TCP.SendMessage( client, statistics.ToString2() );
   if ( ac.ShowStatistics ) {
if ( Globals.IsDebug ) {
 Console.WriteLine( "-> client:" + movableStats.ToString() );
}
    TCP.SendMessage( client, movableStats.ToString() );
   }
   client.Close();
  }
  else if ( tokens [0].Equals( "halt" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "halt\n"
   ////////////////////////////////////////////////////////////////////////////
   if ( ResourceManager.Applications.Count == 0 ) {
    TCP.SendMessage( client, "1\n" );
    client.Close();
    System.Environment.Exit( 0 );
   }
   else {
    TCP.SendMessage( client, "0\n" );
   }
   client.Close();
  }
  else if ( tokens [0].Equals( "stopruntime" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "stopruntime parameters\n"
   ////////////////////////////////////////////////////////////////////////////
   if ( ResourceManager.Applications.Count == 0 ) {
    TCP.SendMessage( client, "0\n" );
    client.Close();
    System.Environment.Exit( 0 );
   }
   else {
    TCP.SendMessage( client, ResourceManager.Applications.Count + "\n" );
    foreach ( DictionaryEntry de in ResourceManager.Applications ) {
     ApplicationConfig ac = (ApplicationConfig) de.Value;
     bool isOK = true;
     foreach ( Node node in ac.Nodes ) {
      try {
       Socket s = TCP.CreateClientSocket( node.Host, node.Port, 1 );
       if ( s == null ) isOK = false;
       else TCP.SendMessage( s, "stopruntime\n" );
      }
      catch {
       isOK = false;
      }
     }
     try {
      TCP.SendMessage( ac.ClientHost, ac.ClientPort, "runtimestopped\n" );
     }
     catch {
      isOK = false;
     }
     if ( isOK ) {
      TCP.SendMessage( client, ac.CommandLine + "  (Stopped)\n" );
     }
     else {
      TCP.SendMessage( client, ac.CommandLine + "  (Can't connect)\n" );
     }
    }
    client.Close();
    System.Environment.Exit( 0 );
   }
  }
  else if ( tokens [0].Equals( "getnodeslist" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "getnodeslist\n"
   ////////////////////////////////////////////////////////////////////////////
   string toSend = ResourceManager.Nodes.Count + "\n";
   for ( int i = 0; i < ResourceManager.Nodes.Count; i++ ) {
    toSend += ResourceManager.Nodes [i] + "\n";
   }
   TCP.SendMessage( client, toSend );
   client.Close();
  }
 }
 
 public static void InitApplication( string[] tokens, Socket client ) {
  string guid = ResourceManager.CurrentApplication++.ToString();
  int numberOfProcessors = int.Parse( tokens [1] );
  string clientHost = tokens [2];
  int clientPort = int.Parse( tokens [3] );
  string commandLine = TCP.ReadLine( client );
  string currentDirectory = TCP.ReadLine( client );
       
  ApplicationConfig ac = new ApplicationConfig( numberOfProcessors );
  ac.ClientHost = clientHost;
  ac.ClientPort = clientPort;
  ac.CommandLine = commandLine;
  ac.ClientSocket = client;
    
  ac.CurrentDirectory = currentDirectory;
  if ( commandLine.IndexOf( "/showstats" ) >= 0 || commandLine.IndexOf( "-showstats" ) >= 0 ) {
   ac.ShowStatistics = true;
  }
  ResourceManager.Applications.Add( guid, ac );

  for ( int i = 0; i < numberOfProcessors; i++ ) {
  
   string host = (string) ResourceManager.Nodes [i % ResourceManager.Nodes.Count];
   string message = "start " + guid + "\n" + ac.CommandLine + "\n" + ac.CurrentDirectory + "\n";
   if ( ac.AssignedNodes [host] == null ) ac.AssignedNodes [host] = 1;
   else ac.AssignedNodes [host] = (int) ac.AssignedNodes [host] + 1;
   TCP.SendMessage( host, ResourceManager.WNPort, message );
   
  }
 }
}
