//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace MCSharp {
 public class Serialization {
  public static void Serialize( System.IO.Stream stream, object o ) {
   new BinaryFormatter().Serialize( stream, o );
  }
  
  public static object Deserialize( MemoryStream stream ) {
  
   BinaryFormatter bf  = new BinaryFormatter();
      
   object          obj = bf.Deserialize ( stream );
      
   return ( obj );   
   
  }

  public static void Pack( SerializationInfo info, StreamingContext context, object o ) {
   Type t = o.GetType();
   FieldInfo[] fi = t.GetFields( BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public );
   for ( int i = 0; i < fi.Length; i++ ) {
    if ( !fi[i].IsNotSerialized ) {
     object val = fi[i].GetValue( o );
     if ( val != null ) info.AddValue( fi[i].Name, val, val.GetType() );
    }
   }
  }

  public static void UnPack( SerializationInfo info, StreamingContext context, object o ) {
   Type t = o.GetType();

   foreach ( SerializationEntry se in info ) {
    object val = se.Value;
    FieldInfo fi = t.GetField( se.Name, BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public );
    fi.SetValue( o, val );
   }
  }
 }
}
