//
// Copyright (C) 2012 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Collections;
using System.Threading;
using MCSharp;
using System.Reflection;

namespace MCSharp {
 public class Session {
  public static string Guid = "";
  public static bool Initiated = false;
  public static bool Finalized = false;
  public static bool Distributed = true;

  public static bool Init( string[] args ) {
         
   ComputationDuration.SetInitDate();
//if ( Globals.IsDebug ) {
   Console.WriteLine("MC#.Runtime, v. "+Assembly.GetExecutingAssembly().GetName().Version);
//}
   DateTime startSessionDate = DateTime.Now;
   MCStatistics.CurrentStatistics = new MCStatistics();

   Session.Distributed = false;

   int np = 1;
   ThreadManager.n  = Environment.ProcessorCount;
   
   string hostName = "";
        
   for ( int i = 0; i < args.Length; i++ ) {
            
    if ( args [i].StartsWith( "/rm:" ) || args [i].StartsWith( "-rm:" ) ) {
     Session.Distributed = true;
     string[] tokens = args [i].Split( ':' );
     Communicator.RMHost = tokens [1];
     Communicator.RMPort = int.Parse( tokens [2] );
     Session.Guid = tokens [3];
     hostName = tokens [4];     
     Communicator.IsOnClusterNode = true;
     Communicator.IsOnClientNode = false;
    }
    else if ( args [i].Equals( "-n"  ) || args [i].Equals ( "/n" ) )  {
     if ( args.Length > i + 1 )
      ThreadManager.n = int.Parse ( args [i+1] );
     else {
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      Console.WriteLine( "Error! Incorrect usage of parameters!" );
      Console.WriteLine( "Hint: It seems that you specified flag '/n' (or '-n') but didn't specified" );
      Console.WriteLine( "the number of processors/cores!" );
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      Environment.Exit( 0 );
     }      
    }  
    else if ( args [i].Equals( "-np" ) || args [i].StartsWith( "/np" ) ) {
     Session.Distributed = true;
     
     if ( args.Length > i + 1 ) 
      np = int.Parse( args [i+1] );
     else {
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      Console.WriteLine( "Error! Incorrect usage of parameters!" );
      Console.WriteLine( "Hint: It seems that you specified flag '/np' (or '-np') but didn't specified" );
      Console.WriteLine( "the number of processors!" );
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      Environment.Exit( 0 );
     }
    }
    else if ( args [i].Equals( "-showstats" ) || args [i].Equals( "/showstats" ) ) {
     Globals.ShowStatistics = true;
    }
    else if ( args [i].Equals( "-withdebug" ) || args [i].Equals( "/withdebug" ) ) {
     Globals.IsDebug = true;
    }
    else if ( args [i].Equals( "-completeboot" ) || args [i].Equals( "/completeboot" ) ) {
     Globals.CompleteBoot = true;
    }
   }
                  
   if ( Session.Distributed ) {
    CommExec.Init();
   }
            
   Thread.Sleep ( 75 );
   
   if ( hostName.Length == 0 ) {   
    string rmhostFileName = Environment.GetEnvironmentVariable ( "HOME" ) +
                            Path.DirectorySeparatorChar + ".mcsharp" + 
                            Path.DirectorySeparatorChar + ".rmhost";
    if ( File.Exists ( rmhostFileName ) ) {    
     FileStream   fstream = new FileStream ( rmhostFileName, FileMode.Open, FileAccess.Read );
     StreamReader sreader = new StreamReader ( fstream );
     hostName = sreader.ReadLine();
     fstream.Close();    
    }
    else        
     hostName = Dns.GetHostByName( Dns.GetHostName() ).AddressList [0].ToString();     
   }
    
   
   if ( Communicator.IsOnClusterNode && Session.Distributed ) {
        
    Communicator.Init ( hostName );
    
    Thread.Sleep ( 100 );
    
    string toSend = "initiated " + Session.Guid + " " + Communicator.Host + " " + Communicator.Port + "\n";
    
    TCP.SendMessage( Communicator.RMHost, Communicator.RMPort, toSend );
        
    Thread.Sleep( Timeout.Infinite );
    Environment.Exit( 0 );
   }
   IPAddress[] addressList = null; // NULL if it's not in distributed mode
   if ( Session.Distributed ) {
    addressList = Dns.GetHostByName( Dns.GetHostName() ).AddressList;
   }
               
   if ( Session.Distributed && (Communicator.RMHost.Equals( "" ) || Communicator.RMPort < 0) ) {
    string home = Environment.GetEnvironmentVariable( "HOME" );
    FileStream fs = new FileStream( home + Path.DirectorySeparatorChar + ".mcsharp" + 
     Path.DirectorySeparatorChar + ".rmport", FileMode.Open, FileAccess.Read );
    StreamReader sr = new StreamReader( fs );
    Communicator.RMPort = int.Parse( sr.ReadLine() );
    fs.Close();
    Communicator.RMHost = addressList [0].ToString();
   }
    
   //Console.WriteLine ( "Before Session.Distributed ... " );
     
   if ( Session.Distributed ) {
        
    Communicator.Init ( hostName );
                          
    Socket client;
    try {  
     client = TCP.CreateClientSocket( Communicator.RMHost, Communicator.RMPort );
    }
    catch {
     Console.WriteLine( "Error! Can't connect to Resource Manager!" );
     Console.WriteLine( " Either cluster hasn't been loaded yet or some fatal errors occured" );
     Console.WriteLine( " Try to run 'mcshalt' and then 'mcsboot'" );
     Communicator.FinalizeIt();
     CommExec.FinalizeIt();
     return false;
    }
                
    string line = "init " + np + " " + addressList [0] + 
     " " + Communicator.Port + "\n" + Environment.CommandLine + "\n" + Environment.CurrentDirectory + "\n";
if ( Globals.IsDebug ) {
    Console.WriteLine( "-> " + Communicator.RMHost + ":" + Communicator.RMPort + ": " + line );
}
    TCP.SendMessage( client, line );
                                    
    TCP.ReadLine( client ); // initstarted\n    
            
    string guid = TCP.ReadLine( client );
            
    client.Close();
    if ( guid.Equals( "error" ) ) {
     Console.WriteLine( "Error: Can't start application!" );
     Console.WriteLine( TCP.ReadLine( client ) );
     Communicator.FinalizeIt();
     CommExec.FinalizeIt();
     return false;
    }
    Session.Guid = guid;

    Monitor.Enter( MCException.lockString );
    Console.WriteLine( "Application Guid: " + guid );
    Monitor.Exit( MCException.lockString );
   }
   else {
    CommWorld.Initialize( 
     new string[] { Communicator.Host + " " + Communicator.Port } );
   }

   if ( Globals.CompleteBoot ) {
    while ( !CommWorld.Initialized ) {
     Thread.Sleep( 10 );
    }
   }
   Session.Initiated = true;
   Session.Finalized = false;
   MCStatistics.CurrentStatistics.SessionInitializationTime = DateTime.Now.Subtract( startSessionDate );
      
   return true;
  }

  public static bool Init() {
   return Init( new string[] {} );
  }

  public static void FinalizeIt() {
   if ( !Initiated || Finalized ) {
    return;
   }
   Finalized = true;
   ComputationDuration.SetEndDate();

   MCStatistics statistics = MCStatistics.CurrentStatistics;
   int maxAddressLength = 0;
   int maxNumberOfCalls = 0;
   int sumNumberOfCalls = 0;
   int numberOfRows = 0;
   int numberOfNodes = 0;
   ArrayList movableStats = new ArrayList();
   if ( Session.Distributed ) {
    string line = "stop " + MCSharp.Session.Guid + "\n";
    Socket client = TCP.SendMessage( Communicator.RMHost, Communicator.RMPort, line );
    string stats = "";
    for ( int i = 0; i < 10; i++ ) {
     stats += TCP.ReadLine( client ) + "\n";
    }
    statistics.Add( new MCStatistics( stats ) );    
    if ( Globals.ShowStatistics ) {
     numberOfNodes = int.Parse( TCP.ReadLine( client ) );
     numberOfRows = int.Parse( TCP.ReadLine( client ) );
     for ( int i = 0; i < numberOfRows; i++ ) {
      string host = TCP.ReadLine( client );
      int number = int.Parse( TCP.ReadLine( client ) );      
      sumNumberOfCalls += number;
      if ( maxAddressLength < host.Length ) maxAddressLength = host.Length;
      if ( maxNumberOfCalls < number ) maxNumberOfCalls = number;
      movableStats.Add( new object[] { host, number } );
     }
    }
   }
   Console.WriteLine( "________________________________________________" );
   Console.WriteLine( "==MC# Statistics================================" );
   Console.Write( statistics );
   TimeSpan ts = ComputationDuration.GetDuration();
   Console.WriteLine( "Session initialization time: " + statistics.SessionInitializationTime +
   " / " + statistics.SessionInitializationTime.TotalSeconds + " sec. / " +
   statistics.SessionInitializationTime.TotalMilliseconds + " msec.");
   Console.WriteLine( "Total time: " + ts +
    " / " + ts.TotalSeconds + " sec. / " + ts.TotalMilliseconds + " msec." );
      
   if ( Session.Distributed && Globals.ShowStatistics ) {
    Console.WriteLine( "Number of movable calls (mc) per node:" );
    int c = 0;
    foreach ( object[] o in movableStats ) {
     c++;
     int len = (int)(((int)o[1]/(double)maxNumberOfCalls)*20);
     StringBuilder sb = new StringBuilder( "" );
     for ( int i = 0; i < len; i++ ) sb.Append( "*" );
     int n = (int) o[1];
     Console.WriteLine( "{0,4}. {1,-" + maxAddressLength + "} | {2,-20} {3, 4} mc {4, 2}%", c, o [0], sb.ToString(), 
      n, (n / (double) sumNumberOfCalls * 100).ToString("F") );
    }
    Console.WriteLine( "--- " + movableStats.Count + " of " + numberOfNodes + " nodes were used ---" );
   }
   Console.WriteLine( "________________________________________________" );
   
   Communicator.FinalizeIt();
   
   CommExec.FinalizeIt();
   Thread.Sleep ( 100 );
      
  }
 }
}
