//
// Copyright (C) 2012 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections;
using System.Diagnostics;
using MCSharp;

class WorkNode {
 public static int Port = 9999;
 public static string Host = "";
 public static string DnsName = "";

 public static int RMPort = 8888;
 public static string RMHost = "";
  
 public static ArrayList Nodes = new ArrayList();
 public static Hashtable Applications = new Hashtable( 10 );

 public static int CurrentApplication = 0;

 public static int bootPort = 10000;

 public static void Usage() {
  Console.WriteLine( "Usage: mono wn.exe /rm:rmHost:rmPort:host" );
  Environment.Exit( 0 );
 }
 
 // wn /rm:host:port
 public static void Main( string[] args ) {
  // Parsing parameters
  if ( args.Length == 0 ) {
   Usage();
  }
  else {
   string[] toks = args [0].Split( ':' );
   try {
    RMHost = toks [1];
    RMPort = int.Parse( toks [2] );
    WorkNode.DnsName = toks [ 3 ];
   }
   catch {
    Usage();
   }
  }
  
  //WorkNode.DnsName = Dns.GetHostName();
  IPAddress[] addressList = Dns.GetHostByName( WorkNode.DnsName ).AddressList;
  WorkNode.Host = addressList [0].ToString();
      
  string home = Environment.GetEnvironmentVariable( "HOME" );
  
  string name = home + Path.DirectorySeparatorChar + ".mcsharp" +
                Path.DirectorySeparatorChar + ".wnport";
  try {
  
   FileStream fs = new FileStream( name, FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   WorkNode.Port = int.Parse( sr.ReadLine() );
   fs.Close();
   
  }
  catch ( Exception ex )
  {
   Console.WriteLine( "File " + name + 
                      " with WorkNode port wasn't found... Setting WN port to 9999!" );
   WorkNode.Port = 9999;
  }

  WorkNode.bootPort = WorkNode.Port + 1;

  string curDir = Environment.CurrentDirectory;
  if ( !curDir.EndsWith( "/" ) ) curDir += "/";
      
  string mcsharpPath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );
  if ( !mcsharpPath.EndsWith( "/" ) ) mcsharpPath += "/";
	  
  try {   
   IPEndPoint ipep = new IPEndPoint( IPAddress.Any, WorkNode.Port );
   Socket serverSocket = new Socket( AddressFamily.InterNetwork, 
    SocketType.Stream, ProtocolType.Tcp );
   serverSocket.Bind( ipep );
   serverSocket.Listen( 1 );
   serverSocket.Close();
  }
  catch ( Exception e ) {
   Console.WriteLine( "EXCEPTION: " + e.ToString() );
   Console.WriteLine( "Error 1! Port " + WorkNode.Port + " is already in use!" );
   Console.WriteLine( " Check your configuration." );
   Console.WriteLine( " If you think that everything is OK there then try to change the WorkNode's" );
   Console.WriteLine( " port number in file '" + home + Path.DirectorySeparatorChar + 
   ".mcsharp" + Path.DirectorySeparatorChar + ".wnport" + "'" );
   return;
  }

  Start();
 }

 public static void Start() {
  IPAddress[] addressList = Dns.GetHostByName( WorkNode.DnsName ).AddressList;
  IPEndPoint ipep = new IPEndPoint( IPAddress.Any, WorkNode.Port );
  Socket serverSocket = new Socket( AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp );
  serverSocket.Bind( ipep );
  serverSocket.Listen( 100 );

  Console.WriteLine( "Started Work Node 2.0 on {0}:{1}", Host, Port );

  while ( true ) {
   try { 
    Socket client = serverSocket.Accept();
    WorkNodeHandler.Handle( client );
   }
   catch ( Exception e ) {
    MCException.WriteException( "RM0002", "", e );
   }
  }
 }
}

public class WorkNodeHandler {
 public static void Handle( Socket client ) {
  string command = TCP.ReadLine( client );
      
if ( Globals.IsDebug ) {
  Console.WriteLine( "WN: " + command );
}

  string[] tokens = command.Split( ' ' );

  if ( tokens [ 0 ].StartsWith ( "ping" ) )  {
  ////////////////////////////////////////////////////////////////////////////
  // Message format: "ping
  ////////////////////////////////////////////////////////////////////////////
   Socket bootSocket = TCP.CreateClientSocket( WorkNode.RMHost, WorkNode.bootPort );
   TCP.SendMessage( bootSocket, "OK\n" );
   
   Thread.Sleep ( 100 );
   bootSocket.Close();
  }
  else if ( tokens [0].Equals( "start" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "start applicationID\n
   // commandLine\n"
   ////////////////////////////////////////////////////////////////////////////
   InitApplication( tokens, client );
  }
  else if ( tokens [0].Equals( "stop" ) ) {
   ////////////////////////////////////////////////////////////////////////////
   // Message format: "stop\n"
   ////////////////////////////////////////////////////////////////////////////
   System.Environment.Exit( 0 );
  }
 }
 
 public static void InitApplication( string[] tokens, Socket client ) {
  string guid = tokens [1];
  string commandLine = TCP.ReadLine( client );
  string currentDirectory = TCP.ReadLine( client );
  StartInfo si = new StartInfo( commandLine, currentDirectory, guid );
  Thread t = new Thread( new ThreadStart( si.Start ) );
  t.Start();
 }
}

public class StartInfo {
 string guid;
 string commandLine;
 string currentDirectory;
 
 public StartInfo( string commandLine, string currentDirectory, string guid ) {
  this.commandLine = commandLine;
  this.guid = guid;
  this.currentDirectory = currentDirectory;
 }
 
 public void Start() {
  string mcsharpPath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );
  if ( !mcsharpPath.EndsWith( "" + Path.DirectorySeparatorChar ) ) 
   mcsharpPath += Path.DirectorySeparatorChar;

  string logPath = "/dev/null";
  
  if ( commandLine.IndexOf( "/withlog" ) >= 0 || commandLine.IndexOf( "-withlog" ) >= 0 ) {
   logPath = "\"" + Environment.GetEnvironmentVariable( "HOME" ) + Path.DirectorySeparatorChar + 
    ".mcsharp" + Path.DirectorySeparatorChar + "logs" + Path.DirectorySeparatorChar +
    "app_" + guid + "_" + WorkNode.DnsName + ".log\"";
  }
  ProcessStartInfo psi = new ProcessStartInfo();
  psi.FileName = mcsharpPath + "bin/starter";
  psi.WorkingDirectory = currentDirectory;
  psi.Arguments = " " + logPath + " mono " +
   commandLine + " /rm:" + WorkNode.RMHost + ":" + WorkNode.RMPort + ":" + guid + ":" + WorkNode.DnsName;
  psi.UseShellExecute = false;  
  Process.Start( psi );  
 }
}
