//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.Runtime.Serialization;
using System.Collections;
using System.Net;

namespace MCSharp 
{
 [Serializable()]
 public class channel : ISerializable 
 {
  public string Host = "";
  public int Port = 0;
  public string MethodName = "";
  public int ObjectNumber = 0;
  public bool CreatedOnClientNode = false;
  [NonSerialized] private Queue Messages;

  public channel( SerializationInfo info, StreamingContext context ) 
  {
   Host = (string) info.GetValue( "0", typeof( string ) );
   MethodName = (string) info.GetValue( "1", typeof( string ) );
   ObjectNumber = (int) info.GetValue( "2", typeof( int ) );
   Port = (int) info.GetValue( "3", typeof( int ) );
   CreatedOnClientNode = (bool) info.GetValue( "4", typeof( bool ) );
  }

  public void GetObjectData( SerializationInfo info, StreamingContext context ) 
  {
   info.AddValue( "0", Host );
   info.AddValue( "1", MethodName );
   info.AddValue( "2", ObjectNumber );
   info.AddValue( "3", Port );
   info.AddValue( "4", CreatedOnClientNode );
  }

  protected channel() {}

  public channel( object o, int objectNumber, string methodName ) 
  {
   this.Host = Communicator.Host;
   if ( this.Host.Equals( "" ) ) {
    this.Host = Dns.GetHostByName( Dns.GetHostName() ).AddressList [0].ToString();
   }
   
   this.Port = Communicator.Port;
   this.ObjectNumber = objectNumber;
   this.MethodName = methodName;
   this.CreatedOnClientNode = Communicator.IsOnClientNode;
   this.Messages = new Queue();
  }

  public void Send( params object[] parameters ) 
  {
   MCSharp.TCP.DeliverChannelMessage( this, parameters );
  }

  public void Enqueue( object o ) 
  {
   Messages.Enqueue( o );
  }

  public object Dequeue() 
  {
   return Messages.Dequeue();
  }

  public int Count { get{ return Messages.Count; } }

  public override string ToString() 
  {
   return "(channel)" +
    "\n  host: " + Host +
    "\n  communicator port: " + Port +
    "\n  object number: " + ObjectNumber +
    "\n  method: " + MethodName;
  }
 }
}
