//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Diagnostics;

public class DoAll {
 public static void Main( string[] args ) {
  string homepath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );
  if ( !homepath.EndsWith( "/" ) ) homepath += "/";
  
  string fileName = homepath + "bin/nodes";
  if ( args.Length == 0 ) {
   Console.WriteLine( "This tool executes the same command on all nodes" );
   Console.WriteLine( "from file " + fileName );
   Console.WriteLine();
   Console.WriteLine( "Usage: do-all command" );
   return;
  }
  string command = String.Join( " ", args );
  Console.WriteLine( command );

  if ( File.Exists( fileName ) ) {
   FileStream fs = new FileStream( fileName, FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   string host = sr.ReadLine();
   
   string accessTypeFile = Environment.GetEnvironmentVariable( "HOME" ) + Path.DirectorySeparatorChar +
     ".mcsharp" + Path.DirectorySeparatorChar + ".accesstype";
   string accessType = "rsh";
   if ( File.Exists( accessTypeFile ) ) {
    using ( StreamReader sr2 = File.OpenText( accessTypeFile ) ) {
     accessType = sr2.ReadLine();
    }
   }
		   	    
   while ( host != null ) {
    if ( !host.StartsWith( "#" ) ) {
     ProcessStartInfo psi = new ProcessStartInfo();
     psi.FileName = accessType;
     psi.UseShellExecute = false;
     psi.Arguments = host + " " + command;
  
     Process p = Process.Start( psi );
     p.WaitForExit();     
    }
    host = sr.ReadLine();
   }
   sr.Close();
   fs.Close();
  }
  else {
   Console.WriteLine( "MC# Runtime Error : Can't find the file " + fileName + ", where cluster nodes must be specified!" );
  }
 }
}
