//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Threading;
using System.Collections;
using System.Collections.Generic;

using MCSharp;

public class mcsboot {

 public static void Main( string[] args ) {

  DateTime dt1 = DateTime.Now;

  string curDir = Environment.CurrentDirectory;
  if ( !curDir.EndsWith( "/" ) ) curDir += "/";

  string mcsharpPath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );
  if ( !mcsharpPath.EndsWith( "/" ) ) mcsharpPath += "/";

  string fileName = mcsharpPath + "bin/nodes";
  
  //    Handle the input parameters
  
  if ( args.Length != 0 && ! args [ args.Length - 1 ].StartsWith ( "-" ) ) {
   fileName = args [ args.Length - 1 ];
   if ( ! fileName.StartsWith( "/" ) )
    fileName = curDir + fileName;
  }

  bool removeDuplicates = false,
       removeNumbers    = false,
       removeFirstName  = false,
       sequentialStart  = false;
       
  for ( int i = 0; i < args.Length; i++ )   {
  
   if ( args [ i ].StartsWith ( "-d" ) )
    removeDuplicates = true;
    
   if ( args [ i ].StartsWith ( "-n" ) )
    removeNumbers = true;
    
   if ( args [ i ].StartsWith ( "-f" ) )
    removeFirstName = true;
    
   if ( args [ i ].StartsWith ( "-seq" ) )
    sequentialStart = true;
  
  }
  if ( !File.Exists( fileName ) ) {
   Console.WriteLine( "Error: Can't find the file " + fileName +
    " whith names of nodes" );
   return;
  }

  string home = Environment.GetEnvironmentVariable( "HOME" );
  int rmPort = 0;
  try {
   FileStream fs = new FileStream( home + Path.DirectorySeparatorChar + ".mcsharp" +
     Path.DirectorySeparatorChar + ".rmport", FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   String temp_string = sr.ReadLine();

   rmPort = int.Parse( temp_string );
   fs.Close();
  }
  catch ( Exception ex )
  {
   Console.WriteLine ( ex.Message );
   Console.WriteLine( "Error : File ~/.mcsharp/.rmport not found ..." );
   return;
  }

  int wnPort = 0;
  try {
   FileStream fs = new FileStream( home + Path.DirectorySeparatorChar + ".mcsharp" +
     Path.DirectorySeparatorChar + ".wnport", FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   wnPort = int.Parse( sr.ReadLine() );
   fs.Close();
  }
  catch {
   Console.WriteLine( "Error : File ~/.mcsharp/.wnport not found ..." );
   return;
  }

  IPAddress address = Dns.GetHostByName( Dns.GetHostName() ).AddressList [0];
  IPAddress[] addressList = Dns.GetHostByName( Dns.GetHostName() ).AddressList;

  string rmHostName = Dns.GetHostName();

  string rmHost = addressList [0].ToString();
  try {
   IPEndPoint ipep = new IPEndPoint( address, rmPort );
   Socket serverSocket = new Socket( AddressFamily.InterNetwork,
   SocketType.Stream, ProtocolType.Tcp );
   serverSocket.Bind( ipep );
   serverSocket.Listen( 1 );
   serverSocket.Close();
  }
  catch ( Exception e ) {
   Console.WriteLine(
    "-----------------------------------------------------------------------------\n" +
    "Cluster has already been loaded or port " + rmPort + " is already in use!\n" +
    "Run 'mcshalt' and then try to launch 'mcsboot' again!\n" +
    "If it doesn't help then change the Resource Manager's\n" +
    "port number in file '" + home + Path.DirectorySeparatorChar +
    ".mcsharp" + Path.DirectorySeparatorChar + ".rmport" + "'.\n" +
    "-----------------------------------------------------------------------------"
   );
   return;
  }

  // Launching RM
  Console.Write( "Initializing Resource Manager..." );
  ProcessStartInfo psi2 = new ProcessStartInfo();
  psi2.FileName = mcsharpPath + "bin/daemonize";
  psi2.UseShellExecute = false;
  psi2.Arguments = " " + mcsharpPath + "bin/starter \"" + home + Path.DirectorySeparatorChar +
    ".mcsharp" + Path.DirectorySeparatorChar + "logs" + Path.DirectorySeparatorChar + "rm.log\"" +
     " \"" + mcsharpPath + "bin/resman\" \"" + fileName + "\"";
  
  Process p = Process.Start( psi2 );
  p.WaitForExit();

  // Checking connection to RM...
  
  try {
   Socket s = TCP.CreateClientSocket( Dns.GetHostName(), rmPort );
   TCP.SendMessage( s, "ping\n" );
   Console.WriteLine( "OK" );
   s.Close();
  }
  catch ( Exception e ) {
   Console.WriteLine( "Failed" );
   Console.WriteLine( "-----------------------------------------------------------------------------" );
   Console.WriteLine( psi2.FileName + psi2.Arguments );
   Console.WriteLine();
   Console.WriteLine( e.ToString() );
   Console.WriteLine( "-----------------------------------------------------------------------------" );
   Environment.Exit( 0 );
  }

  // Launching Work Nodes
  
  Console.WriteLine( "Initializing Work Node Services..." );
  
  List<String> hosts = prepareHostsNames ( fileName, removeDuplicates, removeNumbers, removeFirstName );

  string accessTypeFile = Environment.GetEnvironmentVariable( "HOME" ) + Path.DirectorySeparatorChar +
         ".mcsharp" + Path.DirectorySeparatorChar + ".accesstype";
  string accessType = "rsh";
  if ( File.Exists( accessTypeFile ) ) {
   using ( StreamReader sr = File.OpenText( accessTypeFile ) ) {
    accessType = sr.ReadLine();
   }
  }

  //   Start the WorkNodes through rsh/ssh ( or locally )

  ArrayList startedNodes    = new ArrayList();
  ArrayList workingNodes    = new ArrayList();
  ArrayList start_processes = new ArrayList();

  foreach ( String host in hosts ) {

   ProcessStartInfo psi3 = new ProcessStartInfo();
   psi3.UseShellExecute = false;

   if ( host.StartsWith ( "localhost" ) )   {
    psi3.FileName = mcsharpPath + "bin/daemonize";
    psi3.Arguments = " " + mcsharpPath + "bin/starter \""  +
    home + Path.DirectorySeparatorChar + ".mcsharp" + Path.DirectorySeparatorChar +
    "logs" + Path.DirectorySeparatorChar + "wn_" + host + ".log\" \"" + mcsharpPath +
    "bin/wn\" /rm:" + rmHostName + ":" + rmPort + ":" + Dns.GetHostName();;
   }
   else   {
    psi3.FileName = accessType;
    psi3.Arguments = " " + host + " \"" + mcsharpPath +
    "bin/daemonize\" \"" + mcsharpPath + "bin/starter\" \"" +
    home + Path.DirectorySeparatorChar + ".mcsharp" + Path.DirectorySeparatorChar +
    "logs" + Path.DirectorySeparatorChar + "wn_" + host + ".log\" \"" + mcsharpPath +
    "bin/wn\" /rm:" + rmHostName + ":" + rmPort + ":" + host;
   }

   Process p2 = Process.Start( psi3 );
   
   start_processes.Add(p2);
   if ( host.StartsWith ( "localhost" ) )
    startedNodes.Add ( Dns.GetHostName() );
   else
    startedNodes.Add ( host );
   
   if ( sequentialStart )
    p2.WaitForExit();
   
  }
  
  if ( ! sequentialStart )
  
   //   Wait when the all WorkNodes has been started

   foreach (Process p3 in start_processes)
    p3.WaitForExit();


  // Checking connection to started WN...

  int bootPort = wnPort + 1;

  IPEndPoint boot_ipep    = new IPEndPoint(IPAddress.Any, bootPort);
  Socket bootServerSocket = new Socket(AddressFamily.InterNetwork,
                                    SocketType.Stream, ProtocolType.Tcp);
  bootServerSocket.Bind(boot_ipep);
  bootServerSocket.Listen(1000);

  foreach ( String shost in startedNodes ) {

   try {

    Console.Write ( " " + shost + " ..." );
    Socket s = TCP.CreateClientSocket( shost, wnPort );

    TCP.SendMessage( s, "ping\n" );
    Socket bootClient = bootServerSocket.Accept();
    string clientAnswer = TCP.ReadLine(bootClient);
    Console.WriteLine( clientAnswer );
    s.Close();
    workingNodes.Add( shost );
   }
   catch ( Exception e ) {
    Console.WriteLine( "Failed" );
    Console.WriteLine( "-----------------------------------------------------------------------------" );
    Console.WriteLine( "Host " + shost + " cannot be connected ..." );
    Console.WriteLine();
    Console.WriteLine( e.ToString() );
    Console.WriteLine( "-----------------------------------------------------------------------------" );
   }

  }

  bootServerSocket.Close();

  // Sending list of loaded WNs to RM
  
  string message = "wns " + workingNodes.Count + "\n";
  foreach ( string node in workingNodes ) {
   message += node + "\n";
  }
  TCP.SendMessage( Dns.GetHostName(), rmPort, message );
  Console.WriteLine( "Initialized " + workingNodes.Count + " nodes!" );

  DateTime dt2 = DateTime.Now;
  Console.WriteLine("Boot time = " + (dt2 - dt1).TotalSeconds + " secs.");

 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////
 
 public static List<String> prepareHostsNames ( String fileName, bool removeDuplicates,
                                                bool removeNumbers, bool removeFirstName ) {
  
  String   new_line;
  bool     found;
  
  FileStream   fs = new FileStream   ( fileName, FileMode.Open, FileAccess.Read );
  StreamReader sr = new StreamReader ( fs );
    
  List<String> hosts = new List<String>();
  SortedList   slist = new SortedList();
  
  String line = sr.ReadLine();
  while ( line != null &&  ( line.StartsWith ( "#" ) || line.Equals ( "" ) ) )
   line = sr.ReadLine();
   
  if ( line == null )
   return ( hosts );
    
  if ( removeFirstName )  {
  
   if ( removeDuplicates )  {
   
    found = false;
    while ( ! found && line != null )   {
    
     new_line = sr.ReadLine();
     
     if ( new_line == null )
      line = null;
     else
      if ( ! new_line.StartsWith ( "#" ) && ! new_line.Equals ( "" ) )
       if( ! line.Equals ( new_line ) ) {
        found = true;
        line = new_line;
       }
    
    }
      
   }  
   else 
    if ( line != null ) 
     line = sr.ReadLine();
  
  }   
  
  int index;
  
  while ( line != null )   {
   
   line = line.Trim();
   
   if ( ! line.StartsWith ( "#" ) && ! line.Equals ( "" ) ) {
   
    if ( removeNumbers )  {     
     index = line.IndexOf ( ':' );
     if ( index != -1 )
      line = line.Substring ( 0, index );
    }
     
    if ( removeDuplicates ) {
     if ( ! slist.ContainsKey ( line ) ) 
      slist.Add ( line, null );           
    }
    else   
     hosts.Add ( line );  
   
   }
   
   line = sr.ReadLine();
  
  }
  
  sr.Close();
  fs.Close();
  
  if ( removeDuplicates )
   for ( int j = 0; j < slist.Count; j++ )
    hosts.Add ( (String)slist.GetKey ( j ) );
    
  return ( hosts );
  
 }
 
}
