//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System.IO;
using System;
using System.Collections;
using System.Diagnostics;
using System.Threading;
using System.Text;
using System.Reflection;

public class mcsc {
 public static void Usage() {
  Console.WriteLine( "MC# Compiler, v. " + Assembly.GetExecutingAssembly().GetName().Version );
  Console.WriteLine( "Usage:" );
  Console.WriteLine( " mcsc filename1.mcs ... filenamen.mcs [-o fileout] [/mcs options]" );   
 }
 
 public static void Main( string[] args ) {
  if ( args.Length == 0 ) {
   Usage();
   return;
  }
  if ( args.Length == 1 && ( args[0] == "-version" || args[0] == "--version" ) ) {
   Console.WriteLine( "MC# Compiler, v. " + Assembly.GetExecutingAssembly().GetName().Version );
   return;
  }

  string monoParameters = "";
  string mcsharpPath = Environment.GetEnvironmentVariable( "MCSHARPPATH" );

  if ( mcsharpPath == null ) {
   Console.WriteLine( "Error! Environment variable $MCSHARPPATH is no set!" );
   Console.WriteLine( "       It must contain the full path to MC# installation folder!" );
   System.Environment.Exit( 1 );
  }
  
  ArrayList mcsFiles = new ArrayList();
  ArrayList csFiles = new ArrayList();
  
  for ( int i = 0; i < args.Length; i++ ) {
   if ( args [i].EndsWith( ".mcs" ) ) {
    mcsFiles.Add( args [i] );
   }
   else {
    monoParameters += args [i] + " ";
   }
  }
  string curDir = Environment.CurrentDirectory;
  if ( !curDir.EndsWith( "" + Path.DirectorySeparatorChar ) )
   curDir += Path.DirectorySeparatorChar;

  foreach ( string file in mcsFiles ) {
   string command1 = mcsharpPath + Path.DirectorySeparatorChar + "bin" +
                     Path.DirectorySeparatorChar + "mcsharp-compiler.exe ";
   string inFile = file;
   if ( !inFile.StartsWith( "/" ) )
    inFile = curDir + inFile;
   command1 += "\"" + inFile + "\" ";
   string outFile = inFile.Substring( 0, inFile.Length - 4 ) + ".cs";
   if ( !outFile.StartsWith( "/" ) )
    outFile = curDir + outFile;
   command1 += "\"" + outFile + "\"";
   csFiles.Add( outFile );

   if ( !File.Exists( inFile ) ) {
    Console.WriteLine( "Error! Can't find file " + inFile );
    System.Environment.Exit( 2 );
   }   
  
   ProcessStartInfo psi = new ProcessStartInfo();
   psi.FileName = "mono";
   psi.Arguments = command1;
   psi.UseShellExecute = false;
  
   Process p = Process.Start( psi );
   p.WaitForExit();
  }
  
  // Phase 2
  StringBuilder sb = new StringBuilder();
  sb.Append( "/r:\"" );
  sb.Append( mcsharpPath );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "lib" );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "mcsharp.dll" );
  sb.Append( "," );
  sb.Append( mcsharpPath );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "gpu" );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "lib" );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "GPU.NET.dll" );
  sb.Append( "," );
  sb.Append( mcsharpPath );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "gpu" );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "lib" );
  sb.Append( Path.DirectorySeparatorChar );
  sb.Append( "GPUMath.dll" );
  sb.Append( "\" " );
  sb.Append( monoParameters );
  sb.Append( " " );
  foreach ( string file in csFiles ) {
   sb.Append( "\"" );
   sb.Append( file );
   sb.Append( "\" " );
  }
  
  ProcessStartInfo psi2 = new ProcessStartInfo();
  psi2.FileName = "gmcs";
  psi2.Arguments = sb.ToString();
  psi2.UseShellExecute = false;
  
  Console.WriteLine ( psi2.FileName + " " + psi2.Arguments );
    
  Process p2 = Process.Start( psi2 );
  p2.WaitForExit();  

 }
}
