//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using MCSharp;

public class mcshalt {
 public static void Main( string[] args ) {
  string parameters = String.Join( " ", args );
  string homepath = Environment.GetEnvironmentVariable( "HOME" );
  if ( !homepath.EndsWith( "/" ) ) {
   homepath += "/";
  }
  
  // Reading RM's port number
  string rmFileName = homepath + ".mcsharp/.rmport";
  if ( !File.Exists( rmFileName ) ) {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "Error! Can't find the file " + rmFileName + ", where Resource Manager's\n" +
    "port number must be specified!\n" +
    "-----------------------------------------------------------------------------"
   );
   Environment.Exit( 0 );
  }
  
  FileStream fs = new FileStream( rmFileName, FileMode.Open, FileAccess.Read );
  StreamReader sr = new StreamReader( fs );
  string line = sr.ReadLine();
  int rmport = 0;
  try {
   rmport = int.Parse( line );
  }
  catch {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "Error! Incorrect RM's port format in file " + rmFileName + "!" +
    "-----------------------------------------------------------------------------"
   );
   Environment.Exit( 0 );
  }

  // Reading WN's port number
  string wnFileName = homepath + ".mcsharp/.wnport";
  if ( !File.Exists( wnFileName ) ) {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "Error! Can't find the file " + wnFileName + ", where Work Nodes Service's\n" +
    "port number must be specified!\n" +
    "-----------------------------------------------------------------------------"
   );
   Environment.Exit( 0 );
  }
  
  FileStream fs2 = new FileStream( wnFileName, FileMode.Open, FileAccess.Read );
  StreamReader sr2 = new StreamReader( fs2 );
  string line2 = sr2.ReadLine();
  int wnport = 0;
  try {
   wnport = int.Parse( line2 );
  }
  catch {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "Error! Incorrect WN's port format in file " + wnFileName + "!" +
    "-----------------------------------------------------------------------------"
   );
   Environment.Exit( 0 );
  }
   
  // Getting name of RM host
  string rmhost = Dns.GetHostByName( Dns.GetHostName() ).AddressList [0].ToString();
  Socket s = null;
  Socket s2 = null;
  try {
   // Create 2 sockets - one for requesting list of nodes
   s = TCP.CreateClientSocket( rmhost, rmport, 1 );
   // and the second on for shutting down runtime
   s2 = TCP.CreateClientSocket( rmhost, rmport, 1 );
  }
  catch {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "It seems that there is no Resource Manager running on the (" + Dns.GetHostName() + ":" + rmport + ").\n" +
    "\n" +
    "This indicates that the MC# runtime environment is not operating.\n" +
    "The MC# runtime environment is necessary for the \"mcshalt\" command.\n" +
    "\n" +
    "Please run the \"mcsboot\" command to initiate the MC# runtime\n" +
    "environment. See the MC# documentation for how to invoke \n" +
    "\"mcsboot\" across multiple machines.\n" +
    "-----------------------------------------------------------------------------"
   );
   Environment.Exit( 0 );
  }

  // Get a list of nodes from RM
  ArrayList listOfWns = new ArrayList();
  TCP.SendMessage( s, "getnodeslist\n" );
  int nNodes = int.Parse( TCP.ReadLine( s ) );
  for ( int i = 0; i < nNodes; i++ ) {
   listOfWns.Add( TCP.ReadLine( s ) );
  }
  s.Close();

  if ( args.Length == 0 ) {
   // Halt RM if there's no running applications
   TCP.SendMessage( s2, "halt\n" );
   string ret = TCP.ReadLine( s2 );
   if ( !ret.Equals( "1" ) ) {
    Console.WriteLine( 
     "-----------------------------------------------------------------------------\n" +
     "There're running tasks on the (" + Dns.GetHostName() + ":" + rmport + ")! Runtime wasn't stopped!\n" +
     "If you still want to stop runtime on this cluster use '-force' option.\n" +
     "-----------------------------------------------------------------------------"
    );
    s2.Close();
    Environment.Exit( 0 );
   }
  }
  else {
   TCP.SendMessage( s2, "stopruntime " + parameters + "\n" );
   Console.WriteLine( "-----------------------------------------------------------------------------" );
   int nApps = int.Parse( TCP.ReadLine( s2 ) );
   if ( nApps == 0 ) {
    Console.WriteLine( "There's no running applications! Stopping Runtime..." );
   }
   else {
    Console.WriteLine( "Stopping " + nApps + " application(s)!" );
    for ( int i = 0; i < nApps; i++ ) {
     Console.WriteLine( " " + TCP.ReadLine( s2 ) );
    }
   }
   Console.WriteLine( "-----------------------------------------------------------------------------" );
   s2.Close();
  }
  if ( listOfWns.Count > 0 ) {
   Console.WriteLine( "Stopping Work Node Services..." );
  }
  foreach ( string wnHost in listOfWns ) {
   try {
    Console.Write( " " + wnHost + "..." );
    Socket skt = TCP.CreateClientSocket( wnHost, wnport );
    TCP.SendMessage( skt, "stop\n" );
    Console.WriteLine( "Stopped" );
   }
   catch {
    Console.WriteLine( "Can't connect or node not responding" );
   }
  }
 }
}
