//
// Copyright (C) 2009 Yury Serdyuk
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using MCSharp;

public class mcsnodes {
 public static void Main( string[] args ) {
  string homepath = Environment.GetEnvironmentVariable( "HOME" );
  if ( !homepath.EndsWith( "/" ) ) {
   homepath += "/";
  }
  string fileName = homepath + ".mcsharp/.rmport";
  
  if ( File.Exists( fileName ) ) {
   FileStream fs = new FileStream( fileName, FileMode.Open, FileAccess.Read );
   StreamReader sr = new StreamReader( fs );
   string line = sr.ReadLine();
   int rmport = 0;
   if ( line != null ) {
    try {
     rmport = int.Parse( line );
    }
    catch {
     Console.WriteLine( "Error: incorrect RM's port format in file " + fileName + "!" );
     return;
    }
    // Getting IP
    string rmhost = Dns.GetHostByName( Dns.GetHostName() ).AddressList [0].ToString();
    try {
     Socket s = TCP.CreateClientSocket( rmhost, rmport );
     if ( args.Length == 0 ) {
      TCP.SendMessage( s, "getnodeslist\n" );
      int nNodes = int.Parse( TCP.ReadLine( s ) );
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      Console.WriteLine( "Here's the list of loaded nodes:" );
      for ( int i = 0; i < nNodes; i++ ) {
       Console.WriteLine( " " + i + ":  " + TCP.ReadLine( s ) );
      }
      Console.WriteLine( "-----------------------------------------------------------------------------" );
      s.Close();
     }
     else {
      s.Close();
     }
    }
    catch {
     Console.WriteLine( 
      "-----------------------------------------------------------------------------\n" +
      "It seems that there is no Resource Manager running on the (" + Dns.GetHostName() + ":" + rmport + ").\n" +
      "\n" +
      "This indicates that the MC# runtime environment is not operating.\n" +
      "The MC# runtime environment is necessary for the \"mcsnodes\" command.\n" +
      "\n" +
      "Please run the \"mcsboot\" command to initiate the MC# runtime\n" +
      "environment. See the MC# documentation for how to invoke \n" +
      "\"mcsboot\" across multiple machines.\n" +
      "-----------------------------------------------------------------------------"
     );
    }
   }
  }
  else {
   Console.WriteLine( 
    "-----------------------------------------------------------------------------\n" +
    "Error! Can't find the file " + fileName + ", where Resource Manager's\n" +
    "port number must be specified!\n" +
    "-----------------------------------------------------------------------------"
   );
  }
 }
}
